from sympy import (Symbol, symbols, Matrix, sin, cos, asin, diff, sqrt, S,
                   diag, Eq, hessian, Function, flatten, Tuple, im, pi, latex,
                   dsolve, solve, fraction, factorial, Subs, Number, oo, Abs,
                   N, solveset, atan,Heaviside,sign)

from sympy.physics.mechanics import dynamicsymbols, ReferenceFrame, Point
from sympy.physics.vector import vpprint, vlatex
from ...dynamics import LagrangesDynamicSystem, HarmonicOscillator, mech_comp

from ..elements import MaterialPoint, Spring, GravitationalForce, Disk, RigidBody2D, Damper, PID, Excitation, Force, base_frame, base_origin
from ...continuous import ContinuousSystem, PlaneStressProblem

import base64
import random
import IPython as IP
import numpy as np
import inspect

from .pendulum import Pendulum, PendulumKinematicExct
from .principles import ComposedSystem, NonlinearComposedSystem, base_frame, base_origin, REPORT_COMPONENTS_LIST
from .tmd import TunedMassDamperRelativeMotion
from ...utilities.components.mech import en as mech_comp

from pint import UnitRegistry
ureg = UnitRegistry()

from functools import cached_property, lru_cache

#Patryk
#dane domyślne i numeryczne
class SpringMassSystem(ComposedSystem):
    """Ready to use sample Single Degree of Freedom System with mass on spring
        Arguments:
        =========
            m = Mass
                -Mass of system on spring

            k = Spring coefficient
                -Spring carrying the system

            ivar = symbol object
                -Independant time variable

            qs = dynamicsymbol object
                -Generalized coordinates

        Example
        =======
        A mass oscillating up and down while being held up by a spring with a spring constant k

        >>> t = symbols('t')
        >>> m, k = symbols('m, k')
        >>> qs = dynamicsymbols('z') # Generalized Coordinates
        >>> mass = SDoFHarmonicOscillator(m,k, qs=[z],) # Initialization of LagrangesDynamicSystem instance

        -We define the symbols and dynamicsymbols
        -Kinetic energy T and potential energy v are evaluated to calculate the lagrangian L
        -Reference frame was created with point P defining the position and the velocity determined on the z axis
        -external forces assigned
        -Next we determine the instance of the system using class LagrangeDynamicSystem
        -We call out the instance of the class
        -If necessary assign values for the default arguments


    """
    scheme_name = 'engine.png'
    real_name = 'engine_real.PNG'

    m=Symbol('m', positive=True)
    k=Symbol('k', positive=True)
    ivar=Symbol('t')
    
    z=dynamicsymbols('z')
    
    def __init__(self,
                 m=None,
                 k=None,
                 z=None,
                 ivar=None,
                 **kwargs):

        
        
        if m is not None: self.m = m
        if k is not None: self.k = k
        if ivar is not None: self.ivar = ivar
        if z is not None: self.z = z
        
   
        self.qs = [self.z]

        self._init_from_components(**kwargs)

    @cached_property
    def components(self):

        components = {}
        
        self.material_point = MaterialPoint(self.m, self.z, qs=self.qs)
        self.spring = Spring(self.k, self.z, qs=self.qs)
        
        components['material_point'] = self.material_point
        components['spring'] = self.spring
        
        return components
        
    def symbols_description(self):
        self.sym_desc_dict = {
            self.m: r'mass of system on the spring',
            self.k: r'Spring coefficient ',
        }

        return self.sym_desc_dict

#dane domyślne i numeryczne
class ForcedSpringMassSystem(SpringMassSystem):
    """Ready to use sample Single Degree of Freedom System with mass on spring
        Arguments:
        =========
            m = Mass
                -Mass of system on spring

            k = Spring coefficient
                -Spring carrying the system

            ivar = symbol object
                -Independant time variable

            qs = dynamicsymbol object
                -Generalized coordinates

        Exampled
        =======
        A mass oscillating up and down while being held up by a spring with a spring constant k

        >>> t = symbols('t')
        >>> m, k = symbols('m, k')
        >>> qs = dynamicsymbols('z') # Generalized Coordinates
        >>> mass = SDoFHarmonicOscillator(m,k, qs=[z],) # Initialization of LagrangesDynamicSystem instance

        -We define the symbols and dynamicsymbols
        -Kinetic energy T and potential energy v are evaluated to calculate the lagrangian L
        -Reference frame was created with point P defining the position and the velocity determined on the z axis
        -external forces assigned
        -Next we determine the instance of the system using class LagrangeDynamicSystem
        -We call out the instance of the class
        -If necessary assign values for the default arguments


    """
    scheme_name = 'engine.png'
    real_name = 'engine_real.PNG'


    F=Symbol('F', positive=True)
    g=Symbol('g', positive=True)

    
    def __init__(self,
                 m=None,
                 k=None,
                 F=None,
                 g=None,
                 z=None,
                 ivar=None,
                 **kwargs):


        if m is not None: self.m = m
        if k is not None: self.k = k
        if ivar is not None: self.ivar = ivar
        if z is not None: self.z = z
        if F is not None: self.F = F
        if g is not None: self.g = g


        self.qs = [self.z]

        self._init_from_components(**kwargs)

    @cached_property
    def components(self):

        components = {}  # przerobić na kompozycję
       
        
        self._undamped_trolley = SpringMassSystem(self.m, self.k, self.z)
        self.force = Force(self.F, self.z, qs=self.qs)
        self.gravitational_force = GravitationalForce(self.m, self.g, self.z, qs = self.qs)
        

        components['_undamped_trolley'] = self._undamped_trolley
        components['force'] = self.force
        components['gravitational_force'] = self.gravitational_force
        
        return components


    def symbols_description(self):

        self.sym_desc_dict = {
            self.m: r'mass of system on the spring',
            self.k: r'Spring coefficient ',
        }

        return {**super().symbols_description(),**self.sym_desc_dict}

#dziedziczenie po SpringMass
#owner: KrzychT
#supervision: Boogi
class EquivalentGearModel(ComposedSystem):
    """Ready to use sample Single Degree of Freedom System with mass on spring
        Arguments:
        =========
            m = Mass
                -Mass of system on spring

            k = Spring coefficient
                -Spring carrying the system

            ivar = symbol object
                -Independant time variable

            qs = dynamicsymbol object
                -Generalized coordinates

        Example
        =======
        A mass oscillating up and down while being held up by a spring with a spring constant k

        >>> t = symbols('t')
        >>> m, k = symbols('m, k')
        >>> qs = dynamicsymbols('z') # Generalized Coordinates
        >>> mass = SDoFHarmonicOscillator(m,k, qs=[z],) # Initialization of LagrangesDynamicSystem instance

        -We define the symbols and dynamicsymbols
        -Kinetic energy T and potential energy v are evaluated to calculate the lagrangian L
        -Reference frame was created with point P defining the position and the velocity determined on the z axis
        -external forces assigned
        -Next we determine the instance of the system using class LagrangeDynamicSystem
        -We call out the instance of the class
        -If necessary assign values for the default arguments


    """
    scheme_name = 'engine.png'
    real_name = 'engine_real.PNG'

    m=Symbol('m_{eq}', positive=True)
    k=Symbol('k_g', positive=True)
    F=Symbol('F', positive=True)
    c=Symbol('c_g',positive=True)
    T=Symbol('T',positive=True)
    omega=Symbol('omega',positive=True)
    ivar=Symbol('t')
    k_var=Symbol('kappa_mesh', positive=True)
    eps=Symbol('varepsilon', positive=True)
    c_var=Symbol('c_var', positive=True)
    f=Symbol('f', positive=True)
    
    z=dynamicsymbols('z')
    
    def __init__(self,
                 m=None,
                 k=None,
                 F=None,
                 z=None,
                 c=None,
                 T=None,
                 ivar=None,
                 k_var=None,
                 eps=None,
                 c_var=None,
                 f=None,
                 **kwargs):

        if k_var is not None: self.k_var = k_var
        if eps is not None: self.eps = eps
        if m is not None: self.m = m
        if k is not None: self.k = k
        if F is not None: self.F = F
        if z is not None: self.z = z
        if c is not None: self.c= c
        if T is not None: self.T= T
        if c_var is not None: self.c_var = c_var
        if ivar is not None: self.ivar = ivar
        if f is not None: self.f=f

        
   
        self.qs = [self.z]
        self._init_from_components(**kwargs)

    @cached_property
    def components(self):

        components = {}
        self.c_var = self.c*(1+0*self.eps*self.k_var )
        stiffness = self.k*(1+self.eps*self.k_var  ) 
        
        
        self.gear_inertia = MaterialPoint(self.m, self.z, qs=self.qs)
        self.gear_stiffness = Spring(stiffness, self.z, qs=self.qs)
        self.force = Force(self.F, self.z, qs=self.qs)
        self.gear_damping = Damper(self.c_var, self.z, qs=self.qs)
        
        components['gear_inertia'] = self.gear_inertia
        components['gear_stiffness'] = self.gear_stiffness
        components['force'] = self.force
        components['gear_damping']=self.gear_damping
        
        return components
        
    def symbols_description(self):
        self.sym_desc_dict = {
            self.m: r'mass of the system on spring',
            self.F: r'excitation force',
            self.c: r'damping constant',
            self.k: r'stiffness',
            self.T: r'torque [not sure]',
        }

        return self.sym_desc_dict
    def _report_components(self):

        comp_list = [
            *REPORT_COMPONENTS_LIST
        ]

        return comp_list

    def units(self):
        f=Symbol('f')
        units_dict={
            self.k:ureg.newton/ureg.meter,
            self.m:ureg.kilogram,
            self.F:ureg.newton,
            self.ivar:ureg.second,
            self.T:ureg.newton*ureg.meter,
            self.z.diff(self.ivar,2):ureg.meter/ureg.second**2,
            self.c:ureg.newton*ureg.second/ureg.meter,
            self.f:ureg.hertz
           }
        return units_dict
    
    def get_numerical_data(self):

        k,m,c,F,eps = symbols('k m c F epsilon', positive=True)

        default_data_dict = {
            self.k : [1e6],
            self.m : [1],
            self.c : [2e-4*1e6],
            self.F : [100],
            self.eps : [0.1],
        }
        return default_data_dict
    
    def trig_stiff(self, angle=2*pi):
        trig = sin(self.omega * self.ivar)
        new_eq = self.subs(self.k_var,trig)
        
        return new_eq
    
    def wave_stiff(self):
        wave1=(100*(sin(self.ivar/self.T))+1000)*Heaviside(sin(self.ivar/self.T))
        wave2=(100*(-sin(self.ivar/self.T)))*Heaviside(-sin(self.ivar/self.T))
        waves=wave1+wave2
        new_eq = self.subs(self.k_var,waves)

        return new_eq
        
    def rect_stiff(self,no=6,numerical=False):
        t=self.ivar
        omg = self.omega
        
        trig=sum([Heaviside(omg*t-1) + 0* Heaviside(omg*t-2)  for ind in range(no)])
        new_eq=self.subs(self.k_var,trig)
        
        return new_eq
    
    def approx_rect(self,no=6,numerical=False):
        
        if numerical is True:
            amps_list = [2.27348466531425,0, 0.757408805199249,0, 0.453942816897038,0, 0.323708002807428,0, 0.25121830779797,0, 0.204977919963796,0, 0.172873394602606,0, 0.149252079729775,0, 0.131121653619234,0, 0.116749954968057,0]
        else:
            amps_list = symbols(f'a_0:{no}')
        
        rectangular_approx = sum([N(amp,3)*sin(((ind)+1)*self.omega*self.ivar) for ind,amp in enumerate(amps_list[0:no])])
        new_eq=self.subs({self.k_var:(rectangular_approx)})
        
        return new_eq
    
    def ode_with_delta(self):
        delta=Symbol('delta', positive=True)
        eps =Symbol('varepsilon', positive=True)
        
        with_delta = self._eoms[0]+self.k*eps*delta*self.z
        delta_sys = type(self._ode_system)(with_delta, Matrix([self.z]), ivar=self.ivar, ode_order=2)

        return delta_sys


    
    
    def _stiffness_models(self):
    
        

        #wave
        t = self.ivar
        T = self.T
        
        
        wave1=(1*(sin(2*pi*t/T))+2.0)*(1/2 + 1/2*sign(sin(2*pi*t/T)))
        wave2=(1*(-sin(2*pi*t/T))-3.0)*(1/2+ 1/2*sign(-sin(2*pi*t/T)))
        waves=wave1+wave2



        #rectangular
        rectangular=5*((1/2+1/2*sign(sin(2*pi*t/T)))-S.Half)



        #rectangular_approx
        amps_list = [2.27348466531425, 0.757408805199249, 0.453942816897038, 0.323708002807428, 0.25121830779797, 0.204977919963796, 0.172873394602606, 0.149252079729775, 0.131121653619234, 0.116749954968057]
        rectangular_approx = sum([amp*2/sqrt(2)*sin((2*(no)+1)*2*pi*t/T) for no,amp in enumerate(amps_list[0:])])


        
        return {'wave':waves, 'rect':rectangular, 'approx':rectangular_approx}

    def _stiffness_waveforms(self):
        
        from sympy import lambdify
        
        t = self.ivar
        T = self.T
        
        return {label:lambdify((t, T), waveform)   for label,waveform in self._stiffness_models().items() }
    
    
    
class DDOFGearMechanism(ComposedSystem):
    scheme_name = 'MDOF_Forced_Disks_With_Serial_Springs.PNG'
    real_name = 'three_carriages.PNG'

    r1 = Symbol('r_1', positive=True)
    r2 = Symbol('r_2', positive=True)
    J1 = Symbol('J_1', positive=True)
    J2 = Symbol('J_2', positive=True)
    k = Symbol('k', positive=True)
    c = Symbol('c', positive=True)
    T1 = Symbol('T_1', positive=True)
    T2 = Symbol('T_2', positive=True)
    Omega = Symbol('Omega', positive=True)
    phi1 = dynamicsymbols('phi_1')
    phi2 = dynamicsymbols('phi_2')
    qs = dynamicsymbols('phi_1 phi_2')
    ivar = Symbol('t')

    def __init__(self,
                 r1=None,
                 r2=None,
                 J1=None,
                 J2=None,
                 k=None,
                 c=None,
                 T1=None,
                 T2=None,
                 Omega=None,
                 phi1=None,
                 phi2=None,
                 qs=None,
                 ivar=Symbol('t'),
                 **kwargs):

        if r1 is not None: self.r1 = r1
        if r2 is not None: self.r2 = r2
        if J1 is not None: self.J1 = J1
        if J2 is not None: self.J2 = J2
        if k is not None: self.k = k
        if c is not None: self.c = c
        if T1 is not None: self.T1 = T1
        if T2 is not None: self.T2 = T2
        if Omega is not None: self.Omega = Omega
        if phi1 is not None: self.phi1 = phi1
        if phi2 is not None: self.phi2 = phi2
        if qs is not None: self.qs = qs

        self.ivar = ivar

        self._init_from_components(**kwargs)

    @cached_property
    def components(self):
        components = {}

        self.gear1 = Disk(self.J1, pos1=self.phi1, qs=self.qs, ivar = self.ivar)(label='Drive gear')
        self.gear2 = Disk(self.J2, pos1=self.phi2, qs=self.qs, ivar = self.ivar)(label='Driven gear')


        self.gears_equivalent_stiffness = Spring(self.k, pos1=self.phi1*self.r1, pos2=self.phi2*self.r2, qs=self.qs)(label='Gears equivalent stiffness of teeth')
        self.gears_equivalent_damping = Damper(self.c, pos1=self.phi1*self.r1, pos2=self.phi2*self.r2, qs=self.qs)(label='Gears equivalent damping coefficient of teeth')


        self.torque1 = Force(self.T1 * cos(self.Omega * self.ivar), pos1 = self.phi1, qs = self.qs)(label='Torque of the driving gear')
        self.torque2 = Force(self.T2 * cos(self.Omega * self.ivar), pos1 = self.phi2, qs = self.qs)(label='Torque of the driven gear')


        components['_gear1'] = self.gear1
        components['_gear2'] = self.gear2
        components['_gears_equivalent_stiffness'] = self.gears_equivalent_stiffness
        components['_gears_equivalent_damping'] = self.gears_equivalent_damping
        components['_torque1'] = self.torque1
        components['_torque2'] = self.torque2

        return components

    def symbols_description(self):
        self.sym_desc_dict = {
            self.J1: r'moment of inertia of the driving gear',
            self.J2: r'moment of inertia of the driven gear',
            self.r1: r'rolling radius of the driving gear',
            self.r2: r'rolling radius of the driven gear',
            self.T1: r'torque of the driving gear',
            self.T2: r'torque of the driven gear',
            self.k: r'meshing stiffness',
            self.c: r'meshing damping',
            self.Omega: r'rotation frequency',
        }

        return self.sym_desc_dict

    def units(self):
        units_dict={
            self.J1:ureg.kilogram*ureg.meter**2,
            self.J2:ureg.kilogram*ureg.meter**2,
            self.Omega:ureg.radian/ureg.second,
            self.r1:ureg.meter,
            self.r2:ureg.meter,
            self.ivar:ureg.second,
            self.T1:ureg.newton*ureg.meter,
            self.T2:ureg.newton*ureg.meter,
           }
        return units_dict
    
    def as_equivalent_sdof(self):
        return EquivalentSDOFGearModel()

class EquivalentSDOFGearModel(EquivalentGearModel):
    pass



from ..mechanics import *

import pandas as pd
import numpy as np
from sympy import lambdify

from sympy import *
from pandas import *
from sympy.physics.mechanics import dynamicsymbols

from ....solvers.linear import *
from ....dynamics import *

from ...report import display as ip_display


from ..mechanics import display



miesiace_list = ['styczeń', 'luty', 'marzec','kwiecień','maj','czerwiec','lipiec','sierpień','wrzesień','październik','listopad','grudzień']

srednie_temp_list = [-1.9,-0.8,3.2,9.3,14.6,18,20.1,19.5,14.7,9.3,4.8,0.5]

Eg_dzienne_list_watogodziny_na_metr2 =[600,1000,3000,3800,4800,5400,5300,4900,3300,1700,700,500]

Eg_dzienne_kilowatogodziny_na_metr2 = [0.6,1,3,3.8,4.8,5.3,4.9,3.3,1.7,0.7,0.5]

długosc_dnia_w_miesiacach_godziny = [8.3,10.0,11.8,13.9,15.7,16.7,16.3,14.7,12.7,10.7,8.8,7.8]

data_warunki_atmosferyczne = {'Długość dnia w miesiącu':długosc_dnia_w_miesiacach_godziny,'Dzienne natezenie energii [kWh/m^2]':Eg_dzienne_list_watogodziny_na_metr2,'Średnia temperatura':srednie_temp_list}

df = pd.DataFrame(index = miesiace_list,data = data_warunki_atmosferyczne)

class TitlePageComponent(Environment):
    
    latex_name='titlepage'
    guide_title  = 'WPROWADZENIE DO OBSŁUGI ŚRODOWISKA COCALC'
    
    
    def __init__(self, system=None, options=None, arguments=None, start_arguments=None,
                 **kwargs):
        r"""
        Args
        ----
        options: str or list or  `~.Options`
            Options to be added to the ``\begin`` command
        arguments: str or list or `~.Arguments`
            Arguments to be added to the ``\begin`` command
        start_arguments: str or list or `~.Arguments`
            Arguments to be added before the options
        """

        self.system = system
        self.options = options
        self.arguments = arguments
        self.start_arguments = start_arguments

        
        
        super().__init__(options=options, arguments=arguments, start_arguments=start_arguments,**kwargs)
        
        if self.system is not None:

        
            system = self.system

            
            self.append(NoEscape('\centering'))

            self.append(NoEscape('\\Huge MANUAL DO KRÓTKIEGO KURSU PYTHONA\'A \n \n'))
            self.append(NoEscape(f'\\Huge {self.guide_title} \n \n'))
            
            self.append(Command('vspace',arguments='1cm'))

            
            
            if len(system.q)==1:
                dof_str = 'JEDNYM STOPNIU SWOBODY'
            else:
                dof_str = 'WIELU STOPNIACH SWOBODY'
                
            if system._dissipative_potential==0 or system._dissipative_potential is None:
                damping_str = 'NIETŁUMIONE'
            else:
                damping_str = 'TŁUMIONE'

           
            #self.append(NoEscape(f'\\Large {damping_str} UKŁADY O {dof_str} \n \n'))    
            
            self.append(Command('vspace',arguments='1cm'))
            
            self.append(NoEscape(f'{system._label} \n \n'))
            
            self.append(Command('vspace',arguments='1cm'))
            
            self.append(Command('MyAuthor'))
            self.append(NoEscape(f'\\par'))
            self.append(Command('vspace',arguments='1cm'))
            
            #self.append(Command('vspace',arguments='1cm'))
            #self.append(NoEscape(f'\\protect\\par'))
            #self.append(NewLine())
            self.append(Command('MyDate'))


#obsluga kolaka

class CocalcLoginComponent(ReportComponent):
    
    title="Logowanie do CoCalc"


    def append_elements(self):

        #system = self.reported_object # it's useless in the case of permanent content - it's commented for future usage


        display(ReportText('''Link do CoCalca: [LINK](https://cocalc.com/) '''))

        pic1 = Picture('./dynpy/utilities/components/guides/images/ColacSignIn.png', caption = "Przed Zalogowaniem",width='9cm')

        display(pic1)

        display(ReportText('Teraz należy zalogować się'))

        pic2 = Picture('./dynpy/utilities/components/guides/images/MetLogow.png', caption = "Metody logowania",width='9cm')

        display(pic2)

        pic3 = Picture('./dynpy/utilities/components/guides/images/Zalogowany.png', caption = "Zalogowany",width='9cm')

        display(pic3)

        display(ReportText('''Poprzez tego linka można dołączyć do projektu Ongoing: [ONGOING](https://cocalc.com/app?project-invite=dS62jsbRJvcfj2Mu) '''))
        

#Zakladanie Jupytera        

dynpy_imports_code=(
'''

from sympy import* 
from sympy.physics.mechanics import dynamicsymbols, init_vprinting
from sympy.abc import* 
init_vprinting()
from sympy import S
from dynpy.solvers.linear import ODESystem
import pandas as pd
from dynpy.models.mechanics import Pendulum
from dynpy.models import mechanics

from pylatex import Document, Section, Subsection, Itemize, Package,  HorizontalSpace, Description, Marker, Command
from pylatex.section import Paragraph, Chapter
from pylatex.utils import italic, NoEscape
from dynpy.utilities.adaptable import *
from dynpy.utilities.templates.document import BeamerTemplate, MechanicalCase, EngeneeringDrawingGuide,DevelopmentGuide
from dynpy.utilities.templates.tikz import TikzCaSCStandalone
from dynpy.utilities.report import ReportText, Markdown, Picture, SympyFormula, Frame, ObjectCode, Block, AlertBlock, ExampleBlock
from dynpy.utilities.report import (SystemDynamicsAnalyzer, DataPlot, AccelerationComparison, FFTComparison, ReportEntry,             SimulationalBlock, ReportText, SimulationFFT, DataStorage, Markdown, SummaryTable, SympyFormula, SymbolsDescription, DescriptionsRegistry,             ObjectCode,CurrentContainer)
from dynpy.models import mechanics
import inspect

''')

class JupyterSetUpComponent(ReportComponent):
    
    title="Zakładanie Jupytera"


    def append_elements(self):

        #system = self.reported_object # it's useless in the case of permanent content - it's commented for future usage
        display(ReportText('Posługując sie Jupyterem będziemy tworzyć cele i wykonywali w nich pewne zadania.  '))

        pic9 = Picture('./dynpy/utilities/components/guides/images/PoDodaniuCeli.png', width='9cm')

        display(pic9)

        display(ReportText('Aby dodac nową cele wystarczy kliknąć na "prostokąt" i pod nim pojawią się opcje takie jak Code czy Text'))


        pic10 = Picture('./dynpy/utilities/components/guides/images/Markdown.png', width='9cm')

        display(pic10)

        display(ReportText('Po wybraniu opcji Text będziemy mogli napisać opis zadania, które wykonujemy lub nadać sekcji nagłówek  '))

        pic11 = Picture('./dynpy/utilities/components/guides/images/Plotki-w-naglowk.png', width='9cm')


        display(ReportText('Aby dodać nagłowek wystarczy wpisac znak Hashtag/płotek, ważne jest to że każdy nagłówek będzie traktowany jako sekcja niższego stopnia (\# główna sekcja, \#\# podsekcja itd...)'))

        display(pic11)

        pic12 = Picture('./dynpy/utilities/components/guides/images//Nazywanie-naglowka.png', width='9cm')

        display(pic12)


        display(ReportText('Po wejsciu w zakładkę Edit ukażą nam się poniższe opcje, takie jak usuwanie cel.'))

        pic13 = Picture('./dynpy/utilities/components/guides/images/Del-inne-opcje.png', width='9cm')

        display(pic13)


        display(ReportText('Czasami gdy Jupyter się zawiesi można spróbować go zrestartować uzywając poniższego przycisku'))

        pic14 = Picture('./dynpy/utilities/components/guides/images/Restart.jpeg', width='9cm')

        display(pic14)

        pic15 = Picture('./dynpy/utilities/components/guides/images/Restart-kernel.png', width='9cm')

        display(pic15)

        display(ReportText('Aby mieć pewność że w naszym kodzie uwzględnione zostały wszystkie zaimportowane biblioteki warto uruchamiając jupytera użyć opcji Run all cells pokazanej na poniższych obrazkach'))

        pic16 = Picture('./dynpy/utilities/components/guides/images/RunAll.jpeg', width='9cm')

        display(pic16)

        pic17 = Picture('./dynpy/utilities/components/guides/images/Run_all.png', width='9cm')

        display(pic17)

        display(ReportText('Aby elementy z których korzystamy działay należy zaimportować potrzebne biblioteki, takie jak na załączonym obrazku oraz w formie tekstu który można skopiowac'))

        pic18 = Picture('./dynpy/utilities/components/guides/images/ImportowanieBibliotek.png', width='9cm')

        # display(pic18)

        display(ObjectCode(dynpy_imports_code))


class CocalcFolderComponent(ReportComponent):
    
    title="Tworzenie Folderu i Jupytera"


    def append_elements(self):
        
        #system = self.reported_object # it's useless in the case of permanent content - it's commented for future usage

        display(ReportText('Tworzenie Folderu i Jupytera'))
        
        pic4 = Picture('./dynpy/utilities/components/guides/images/OtwarcieOngo.png', width='9cm')
        
        display(pic4)
        
        display(ReportText('W wyszukiwarce szukamy "Ongoing" i otwieramy projekt  '))
        
        pic5 = Picture('./dynpy/utilities/components/guides/images/PoWejsciuWOngo.png', width='9cm')
        
        display(pic5)
        
        display(ReportText('Otworzenie folderu UCZESTNICY  '))
        
        
        display(ReportText('Nadajemy nazwę folderu od naszego imieniem i nazwiskiem jak na zalączonym obrazku i klikamy create folder:'))
        
        pic6 = Picture('./dynpy/utilities/components/guides/images/NazwanieFolderu.png', width='9cm')
        
        display(pic6)
        
        
        display(ReportText('Po utworzeniu folderu klikamy przycisk NEW i utworzymy pierwszego Jupytera. W tym celu po nazwaniu Jupytera wybieramy zakladke Jupyter Notebook.'))
        
        
        pic7 = Picture('./dynpy/utilities/components/guides/images/WyborKernela.png', width='9cm')
        
        display(pic7)
        
        display(ReportText('Wybieramy Python 3 (system - wide)  '))
        display(ReportText('Po wybraniu Kernela utworzony zostanie nasz Jupyter.'))
        
        pic8 = Picture('./dynpy/utilities/components/guides/images/JupekUtworzony.png', width='9cm')
        
        display(pic8)
        ####

#podstawy raportowania
        



        
code_dynsys_list_str = '''
#komendy importujące biblioteki
import sympy 
from sympy import Symbol

from dynpy.models.mechanics.pendulum import Pendulum

#komenda wywołująca preview
Pendulum().preview()

#komenda wywołująca pomocne informacje dotyczące wahadła
help(Pendulum)

'''



class CocalcDynSysListComponent(ReportComponent):
    
    title="Lista systemów dynamicznych"


    def append_elements(self):

        #system = self.reported_object # it's useless in the case of permanent content - it's commented for future usage
        
        display(ReportText(' Możemy również wyswietlić podgląd takiego wahadła używając poniższej funkcji:  '))
        display(Picture('./dynpy/utilities/components/guides/images/preview.jpg'))
        display(ReportText(' Co więcej, używając poniższej komendy help, możemy otrzymać wszelkie informacje na temat konkretnego modelu:  '))
        display(Picture('./dynpy/utilities/components/guides/images/help.jpg'))

        display(ReportText('''Kod wywołania pomocy jest następujący:'''))
        
        display(ObjectCode(   code_dynsys_list_str   ))


        display(ReportText('Poniżej znajduje się lista systemów mechanicznych których można użyć:  '))

        from dynpy.models import mechanics


        moduly=inspect.getmembers(mechanics, inspect.ismodule)


        for nazwa,modul in moduly:

            mods_tmp = [name for name,cls in inspect.getmembers(modul, inspect.isclass) if issubclass(cls,mechanics.principles.ComposedSystem)]
            classes_str=',\n - '.join(mods_tmp)
            if mods_tmp != []:
                display(ReportText( '\\par' + f'-{nazwa}'+ '\\par'))
                display(ReportText(f'- {classes_str}'))  


class ReportingBasicsComponent(ReportComponent):
    
    title="Podstawy raportowania"


    def append_elements(self):
        from dynpy.utilities.templates.document import Guide
        
        #system = self.reported_object # it's useless in the case of permanent content - it's commented for future usage
        
        display(ReportText('Pierwszy krokiem jest import "szablonu" dokumentu np. klasy `Guide` oraz wywołanie metody `base_setup`. Wynik wywołania jest następujący:'))

        display(ObjectCode(
'''
from dynpy.utilities.templates.document import *
Guide.base_setup()
'''
        ))
        
        display(Guide.base_setup())

        display(ReportText('Metoda zwraca przykładowe cele gotowe do przekopiowania do notatnika.'))
                
        

        
#obsluga systemow dynamicznych


imports_code_str = '''
from sympy import *
import numpy as np
from pandas import *
from sympy.physics.mechanics import init_vprinting, dynamicsymbols
from pylatex import Document, Section, Subsection, Itemize, Package, HorizontalSpace, Description, Marker, Command 
from pylatex.section import Paragraph, Chapter 
from pylatex.utils import italic, NoEscape 
from dynpy.utilities.adaptable import *
from dynpy.utilities.templates.document import BeamerTemplate, MechanicalCase 
from dynpy.utilities.templates.tikz import TikzCaSCStandalone
from dynpy.utilities.report import (SystemDynamicsAnalyzer, DataPlot, AccelerationComparison, FFTComparison, ReportEntry, SimulationalBlock, ReportText, SimulationFFT, DataStorage, Markdown, SummaryTable, Picture, SympyFormula, SymbolsDescription, DescriptionsRegistry, ObjectCode, CurrentContainer)
from dynpy.solvers.linear import ODESystem
init_vprinting()
from dynpy.models.mechanics.tmac import SDOFWinchSystem

'''



class DynamicSystemCallComponent(ReportComponent):
    
    title="Wprowadzenie"


    def append_elements(self):

        #system = self.reported_object # it's useless in the case of permanent content - it's commented for future usage
        display(ReportText('Wymagane importy bibliotek/poszczególnych klas do stworzenia raportu:'))


        display(ObjectCode(imports_code_str))


        display(ReportText('Klasa umożliwiająca rozwiązanie konkretnej problematyki (w tym przypadku problematyki związanej z opisem dźwigu):'))

        display(ObjectCode(
        """
        from dynpy.models.mechanics.tmac import SDOFWinchSystem 
        """))

        display(ReportText('Ścieżka do poszukiwanej klasy na platformie CoCalc:'))
        display(Picture('./dynpy/utilities/components/guides/images/sciezka_w.jpg'))

        display(ReportText('Sposób wywowałania preview klasy - tzw. podglądu:'))

        display(ObjectCode('''
        SDOFWinchSystem().preview()
        '''))
        
        
class DynamicSystemMethodsUsageComponent(ReportComponent):

    title="Wywolywanie rownan ruchu oraz innych metod"


    def append_elements(self):
        
        system = self.reported_object # it's useless in the case of permanent content - it's commented for future usage
        
        #from dynpy.models.mechanics import ForcedSpringMassSystem as SDOFWinchSystem

        eoms=system._eoms[0]
        eoms

        display(ReportText('Proces wywołowania równania ruchu za pomoca metody eoms:'))
#         display(ObjectCode('''eoms=SDOFWinchSystem()._eoms[0]
#         eoms'''
#         ))



        display(ReportText('Wynik jest następujący:'))
        #display(SympyFormula(eoms))


        eoms_eq=Eq(eoms,0)
        eoms_eq

        display(ReportText('Proces tworzenia równania z wcześniej wywołanego równania ruchu (równanie początkowo jest wywoływane jako ciąg znaków reprezentujący opis danego problemu, jednak nie jest on wówczas równaniem, które możemy zastosować do obliczeń):'))
#         display(ObjectCode('''eoms_eq=Eq(eoms,0)
#         eoms_eq'''
#         ))
        display(ReportText('Wynik jest następujący:'))
        display(SympyFormula(eoms))

        #display(Picture('./Image/eomseq_w.jpg'))

        slownik=system.get_random_parameters()

        

        display(ReportText('Tworzenie słownika (w tym przypadku z losowymi wartościami) umożliwiający nadanie wartości zmiennym:'))

        display(Markdown(
"""

    slownik=SDOFWinchSystem().get_random_parameters()
    slownik

"""
        ))
        #display(Picture('./Image/slownik_w.jpg'))

        display(ReportText('Rozwiązanie ogólne równania:'))
        #display(Picture('./Image/ogolne_w.jpg'))

        solution=system._ode_system.solution[0]
#         display(ObjectCode('''solution=SDOFWinchSystem()._ode_system.solution[0]
#         solution'''
#         ))
        display(SympyFormula(solution))

        display(ReportText('Rozwiązanie szczególne równania:'))



        steady_solution=system._ode_system.steady_solution[0]
#         display(ObjectCode('''steady_solution=SDOFWinchSystem()._ode_system.steady_solution[0]
#         steady_solution'''))
        display(SympyFormula(steady_solution))
    
    
    
steady_sol_str=(
'''

steady_solution_subs=steady_solution.subs(slownik)
steady_solution_subsS

''')

solution_table_str=(
'''

table=table_eq.numerized().compute_solution(t_span)
table_new=table[[phi]]

''')

report_table_define_str=(
'''

tabela = Section('Tabela')
CurrentContainer(tabela)

table_eq=general_sol_matrix.n().numerized().compute_solution(t_span)
table=table_eq[[phi]]

table_for_report = table[0::15].to_latex_dataframe()

#Wyniki symulacji zostały zestawione w tabeli

table_for_report.reported(caption=('Tabela'))

''')

class SimulationsComponent(ReportComponent):
    
    title="Tworzenie wykresów"

    def append_elements(self):
        
        system = self.reported_object # it's useless in the case of permanent content - it's commented for future usage


        display(ReportText('Definicja wektoru czasu:'))
        #display(Picture('./Image/tspan_w.jpg'))

        display(ObjectCode('''t_span = np.linspace(0,100,200)'''))
        t_span = np.linspace(0,100,200)


        display(ReportText('Sposób tworzenia wykresów z tak zwanego palca:'))
        display(ReportText('Podstawienie danych z wcześniej stworzonego słownika:'))
        #display(Picture('./Image/stedisub_w.jpg'))

        display(Markdown(steady_sol_str))
        slownik=system.get_random_parameters()
        steady_solution=system._ode_system.steady_solution[0]
        steady_solution_subs=steady_solution.subs(slownik)
        display(steady_solution_subs)
        display(ReportText('Zastosowanie metody lambdify umożliwiającej konwersje funkcji do postaci anonimowej'))
        #display(Picture('./Image/lambdify_w.jpg'))


        eq_lambdify=lambdify(system.ivar,steady_solution_subs.simplify())
        display(ObjectCode('''eq_lambdify=lambdify(system.ivar,steady_solution_subs.simplify())'''))
        display(SympyFormula(eq_lambdify))


        display(ReportText('Stworzenie wykresu:'))
        #display(Picture('./Image/plot2_w.jpg'))
        pd.DataFrame(index=t_span, data=eq_lambdify(t_span)).plot()
        display(ObjectCode('''pd.DataFrame(index=t_span, data=eq_lamdify(t_span)).plot()'''))


        display(ReportText('Sposób tworzenia wykresów bezpośrednio z wcześniej wywołanej klasy:'))
        display(ReportText('Podstawienie słownika:'))
        #display(Picture('./Image/tableq_w.jpg'))

        table_eq=system._ode_system.steady_solution.subs(slownik)


        display(Markdown('''table_eq=system._ode_system.steady_solution.subs(slownik)'''))
        display(SympyFormula(table_eq))

        #jak tabele zrobic + ew jej output
        display(ReportText('Sposób tworzenia tabeli:'))
        #display(Picture('./Image/tabela_w.jpg'))
        phi = system.phi
        table=table_eq.numerized().compute_solution(t_span)
        table_new=table[phi]

        display(ObjectCode(solution_table_str))
        display(table)
        display(table_new)

        display(ReportText('Sposób tworzenia wizualizacji zdefiniowanej tabeli:'))
        #display(Picture('./Image/tabelka_w.jpg'))

        display(ObjectCode('''table_new'''))

        display(ReportText('sposób tworzenia tabeli wyświetlanej w gotowym raporcie:'))
        #display(Picture('./Image/tworzenie_tabeli_w.png'))

        display(Markdown(report_table_define_str))

        display(ReportText('Wizualizacja wykresu::'))
        #display(Picture('./Image/tikzplot_w.jpg'))
        display(table_new.to_pylatex_tikz().in_figure())
        display(ObjectCode('''table_new.to_pylatex_tikz().in_figure()'''))    

section_development_str=(
'''
intro=Section('Wprowadzenie')
CurrentContainer(intro)
display(ReportText('schemat układu'))
schemat=Picture('picture_winch.png')
display(schemat)
''')

forming_equation_str=(
'''
solution_subs = SDOFWinchSystem()._ode_system.steady_solution.subs(slownik)
eq_sol=Eq(solution_subs,0)
eq_steady_sol=Eq(steady_solution_subs,0)

rownania_ruchu=Section('Równania ruchu')
CurrentContainer(rownania_ruchu)
display(ReportText('równania ruchu: '))

display(SympyFormula(eoms_eq))
display(SympyFormula(eq_sol))
display(SympyFormula(eq_steady_sol))
''')

plot_generation_str=(
"""

""")

report_generation_libraries=(
'''
doc_final = MechanicalCase('./Out/Document',documentclass=NoEscape('article'),document_options=['a4paper','fleqn'],lmodern=False)

doc_final.append(intro)
doc_final.append(rownania_ruchu)
doc_final.append(wykres)

doc_final.generate_pdf()
''')

class SimulationReportComponent(ReportComponent):
    
    title="Raportowanie wykresów"


    def append_elements(self):
        
        system = self.reported_object # it's useless in the case of permanent content - it's commented for future usage

        display(ReportText('Sposób tworzenia sekcji przy użyciu metody CurrentContainer:'))

        #sekcja 1|
        display(ObjectCode(section_development_str))


        display(ReportText('Poniżej przedstawiony jest sposób tworzenia równań. W tym przypadku przyrównamy wyrazy (przed przecinkiem) do 0 (po przecinku).'))
        display(ObjectCode(forming_equation_str))



        display(ReportText('poniżej przedstawiony jest sposób tworzenia wykresu dla podanych danych: '))
        eoms=system._eoms[0]
        slownik=system.get_random_parameters()
        eoms_eq=Eq(eoms,0)
        solution_subs = system._ode_system.steady_solution.subs(slownik)
        steady_solution=system._ode_system.steady_solution[0]
        steady_solution_subs=steady_solution.subs(slownik)
        steady_solution_subs
#         eq_sol=Eq(solution_subs,0)
#         eq_steady_sol=Eq(steady_solution_subs,0)
        display(SympyFormula(solution_subs))

#         ode_system = ODESystem.from_dynamic_system(system.linearized())
#         param_dict = system.get_numerical_parameters()
        t_span = np.linspace(1,100,101)
        ic_list = [0.0,0.0] ### Dla układu o jednym stopniu swobody
#         ode_solution = ode_system.subs(param_dict).steady_solution
#         display(SympyFormula(ode_solution))

        ode_simulation = steady_solution_subs.compute_solution(t_span, ic_list = ic_list)#ode_solution.subs(param_dict).compute_solution(t_span, ic_list = ic_list)
    
        display(ReportText('Wykres: '))
        picture=ode_simulation.to_pylatex_tikz(subplots=True).in_figure(caption='Otrzymany wykres')

        display(picture)

        display(ReportText('równania ruchu: '))

        display(SympyFormula(eoms_eq))
        display(SympyFormula(solution_subs))
        display(SympyFormula(steady_solution_subs))




        display(ObjectCode(report_generation_libraries))



        display(ReportText('Tak wygląda ostateczny raport:'))
        display(Picture('./dynpy/utilities/components/guides/images/sekcja3_w.jpg'))
        
        
        
#pandas guide
data_code=(
'''
miesiace_list = ['styczeń','luty','marzec','kwiecień','maj','czerwiec','lipiec','sierpień','wrzesień','październik','listopad','grudzień']  
srednie_temp_list = [-1.9,-0.8,3.2,9.3,14.6,18,20.1,19.5,14.7,9.3,4.8,0.5]  
Eg_dzienne_list_watogodziny_na_metr2 =[600,1000,3000,3800,4800,5400,5300,4900,3300,1700,700,500]  
Eg_dzienne_kilowatogodziny_na_metr2 = [0.6,1,3,3.8,4.8,5.3,4.9,3.3,1.7,0.7,0.5]
''')

dict_code=(
'''
data_warunki_atmosferyczne = {'Długość dnia w miesiącu':długosc_dnia_w_miesiacach_godziny,'Dzienne natezenie energii         [kWh/m^2]':Eg_dzienne_list_watogodziny_na_metr2,'Średnia             temperatura':srednie_temp_list}
''')
output_code=(
'''
df = pd.DataFrame(index = miesiace_list,data = data_warunki_atmosferyczne)
df
''')

class PandasTableGenerationComponent(ReportComponent):
    
    title="Tworzenie tabelki"


    def append_elements(self):
        
        #system = self.reported_object # it's useless in the case of permanent content - it's commented for future usage


        display(ReportText('Stworzenie listy danych:'))

        display(ObjectCode(data_code))


        display(ReportText('Stworzenie słownika:'))

        display(ObjectCode(dict_code))


        display(ReportText('Wywolanie tabelki:'))
        display(ObjectCode(output_code))
        display(df)
        

winch_pandas_code=(
'''

from dynpy.models.mechanics.tmac import SDOFWinchSystem
eoms= SDOFWinchSystem()._eoms[0]
eoms_eq_raw=Eq(eoms,0)
data=SDOFWinchSystem().get_random_parameters()
Ms0=Symbol('M_s0')
b=Symbol('b')

winch = SDOFWinchSystem()

Ms=winch.M_s
t = winch.ivar

dane={
    Ms0:4700,
    b:1500,
    SDOFWinchSystem().m0:10}



winch_up=SDOFWinchSystem().subs(Ms,Ms0-b*SDOFWinchSystem().phi.diff()).subs(data).subs(dane)
nowe_ode=winch_up._ode_system
solution=nowe_ode.solution
steady_sol=nowe_ode.steady_solution[0]
t_span = np.linspace(0,100,200)
phi=winch.q[0]



units_dict = {phi : ureg.radian,phi.diff(t):ureg.radian,t:ureg.second }
LatexDataFrame.set_default_units(units_dict)
''')        
        
class BasicSymComponent(ReportComponent):
    
    title="Symulacje - wariant podstawowy"


    def append_elements(self):
        
        system = self.reported_object # it's useless in the case of permanent content - it's commented for future usage


        #przekopiowac tu fragment kodu z guide pamietajac o indentach
        SDOFWinchSystem = type(system)
#         from dynpy.models.mechanics.tmac import SDOFWinchSystem
        eoms= SDOFWinchSystem()._eoms[0]
        eoms_eq_raw=Eq(eoms,0)
        data=SDOFWinchSystem().get_random_parameters()
        Ms0=Symbol('M_s0')
        b=Symbol('b')

        winch = SDOFWinchSystem()

        Ms=winch.M_s
        t = winch.ivar

        dane={
            Ms0:4700,
            b:1500,
            SDOFWinchSystem().m0:10}



        winch_up=SDOFWinchSystem().subs(Ms,Ms0-b*SDOFWinchSystem().phi.diff()).subs(data).subs(dane)
        nowe_ode=winch_up._ode_system
        solution=nowe_ode.solution
        steady_sol=nowe_ode.steady_solution[0]
        t_span = np.linspace(0,100,200)
        phi=winch.q[0]



        units_dict = {phi : ureg.radian,phi.diff(t):ureg.radian,t:ureg.second }
        LatexDataFrame.set_default_units(units_dict)
        display(ObjectCode(winch_pandas_code))


        display(ReportText('Analiza i symulacje klasy SDOFWinchSystem, tzw. wciągarki. Równania ruchu: '))
        display(SympyFormula(eoms_eq_raw))
        display(ReportText(f'Po podstawieniu danych do ogólnej postaci (do równania {AutoMarker(eoms_eq_raw)}) otrzymuje się: '))
        display(SympyFormula(Eq(winch_up._eoms[0],0)))
        display(ReportText('Rozwiązanie szczególne: '))
        display(SympyFormula(Eq(phi,steady_sol)))
        display(ReportText('Rozwiązanie ogólne: '))
        display(SympyFormula(Eq(phi,solution[0])))
        general_sol_matrix=nowe_ode.solution.with_ics([0,0])
        table_eq=general_sol_matrix.n().numerized().compute_solution(t_span)
        table=table_eq[[phi.diff(t)]]
        table_for_report = table.iloc[0::15].to_latex_dataframe()
        display(Markdown(f'Wyniki symulacji zostały zestawione w tabeli {AutoMarker(table_for_report)}'))
        display(table_for_report.reported(caption='Tabela'))
        sim_plot = table.to_pylatex_tikz().in_figure(caption='Wykres')
        display(Markdown(f'Wyniki symulacji zostały przedstawione graficznie na wykresie {AutoMarker(sim_plot)}'))
        display(sim_plot)
        
        
class PandasMethodsComponent(ReportComponent):
    
    title="Metody biblioteki Pandas"


    def append_elements(self):
        
        #system = self.reported_object # it's useless in the case of permanent content - it's commented for future usage
        
        
        miesiace_list = ['styczeń', 'luty', 'marzec','kwiecień','maj','czerwiec','lipiec','sierpień','wrzesień','październik','listopad','grudzień']

        srednie_temp_list = [-1.9,-0.8,3.2,9.3,14.6,18,20.1,19.5,14.7,9.3,4.8,0.5]

        Eg_dzienne_list_watogodziny_na_metr2 =[600,1000,3000,3800,4800,5400,5300,4900,3300,1700,700,500]

        Eg_dzienne_kilowatogodziny_na_metr2 = [0.6,1,3,3.8,4.8,5.3,4.9,3.3,1.7,0.7,0.5]

        długosc_dnia_w_miesiacach_godziny = [8.3,10.0,11.8,13.9,15.7,16.7,16.3,14.7,12.7,10.7,8.8,7.8]

        data_warunki_atmosferyczne = {'Długość dnia w miesiącu':długosc_dnia_w_miesiacach_godziny,'Dzienne natezenie energii [kWh/m^2]':Eg_dzienne_list_watogodziny_na_metr2,'Średnia temperatura':srednie_temp_list}

        df = pd.DataFrame(index = miesiace_list,data = data_warunki_atmosferyczne)

        display(ReportText('***Metoda iloc:***'))
        display(ObjectCode('df.iloc[0:3]'))
        display(df.iloc[0:3])

        # display(Picture('./Image/iloc.jpg', caption = ""))

        display(ReportText('***Metoda loc:***'))
        display(ObjectCode('df.loc["styczeń"]'))
        display(df.loc["styczeń"])
        # display(Picture('./Image/loc.jpg', caption = ""))

        display(ReportText('***Metoda set axis dziala za rowno dla kolumn jak i rzędów. Dla rzędów:***'))
        display(ObjectCode("def.set_axis(['a','b','c','d','e','f','g','h','i','j','k','l'],axis = 'index')"))
        display(df.set_axis(['a','b','c','d','e','f','g','h','i','j','k','l'], axis = 'index'))

        # display(Picture('./Image/set_axis.jpg', caption = ""))

        display(ReportText('***Dla kolumn:***'))
        display(ObjectCode("df.set_axis(['a','b','c'],axis = 'columns')"))
        display(df.set_axis(['a','b','c'],axis = 'columns'))
        # display(Picture('./Image/set_axis2.jpg', caption = ""))

        display(ReportText('***Metoda rename - Zmienienie pojedynczej kolumny:***'))
        display(ObjectCode('''df.rename(columns = {"Długość dnia w miesiącu":'AAA'}, index = {"styczeń":'A'} )'''))
        display(df.rename(columns = {"Długość dnia w miesiącu":'AAA'}, index = {"styczeń":'A'} ))
        # display(Picture('./Image/rename.jpg', caption = ""))

        display(ReportText('***Metoda applymap:***'))
        display(ObjectCode('     df.applymap(lambda x:x+2)'))
        display(df.applymap(lambda x:x+2))

        # display(ReportText('***Metoda applymap dla kolumny/wiersza:***'))
        # display(Markdown('''
        # Warto pamiętać, że df[NAZWA] => tworzy serię, a df[[Nazwa]] => tworzy nowy frame, który można wykorzystać do użycia .applymap'''))
        # display(df[['Długość dnia w miesiącu']].applymap(lambda x:x+100))
        # pd_h = df[['Długość dnia w miesiącu']].applymap(lambda x:x+100)
        # list1 = pd_h['Długość dnia w miesiącu'].tolist()
        # df['Długość dnia w miesiącu'] = list1

        # display(df)

        # display(Picture('./Image/applymap.jpg', caption = ""))

        display(ReportText('***Slicowanie:***'))
        display(ObjectCode('''df['Długość dnia w miesiącu']'''))
        display(df['Długość dnia w miesiącu'])
        # display(Picture('./Image/slice2.jpg', caption = ""))
        
        
steady_state_code=(
'''
steady_state = SDOFWinchSystem().eoms.solution[0]
steady_state
''')
    
list_code=(
'''
Model_number = [1,2,3,4] 
inertia_list= [1,2,3,4] 
stiffness_list = [1,2,3,4] 
damping_list = [1,2,3,4] 
Excitation_list = [1,2,3,4]

system_parameters = {'Inertia':inertia_list, 'Stiffness':stiffness_list,'Damping':damping_list,'Excitation':Excitation_list}
''')

df_code=(
'''
df = pd.DataFrame(index = Model_numbber, data = system_parameters)
df
''')

dict_code=(
'''
data_dict_1 = df.T[1].to_dict()
data_dict_2 = df.T[2].to_dict()
data_dict_3 = df.T[3].to_dict()
data_dict_4 = df.T[4].to_dict()
eq1 = steady_state.subs(data_dict_1).subs(SDOFWinchSystem().F,100)
eq2 = steady_state.subs(data_dict_2).subs(SDOFWinchSystem().F,100)
eq3 = steady_state.subs(data_dict_3).subs(SDOFWinchSystem().F,100)
eq4 = steady_state.subs(data_dict_4).subs(SDOFWinchSystem().F,100)
''')



sym_gen_sol=(
'''
    table = SDOFWinchSystem().subs(data_dict_2).subs(SDOFWinchSystem.F,100).numerized().compute_solution(t_span,[0,0]).iloc[:,0] 
    table.plot() 
''')

list_elem = (
'''
eq2_fun = lambdify(SDOFWinchSystem().ivar,eq2.simplify())
eq2_fun(t_span)
''')


solution_code=(
'''
eq2_fun = lambdify(SDOFWinchSystem().ivar,eq2.simplify())
eq2_fun(t_span)
pd.DataFrame(index = t_span, data = eq2_fun(t_span).plot()
''')



class DifferentSimulationsComponent(ReportComponent):
    
    title="Symulacje - pozostałe warianty"



    
    def append_elements(self):
        
        system = self.reported_object # it's useless in the case of permanent content - it's commented for future usage
        
        SDOFWinchSystem = type(system)

        display(ReportText('Wygenerowanie rozwiazania szczegolnego klasy:'))
        
        display(ObjectCode(   steady_state_code.replace('SDOFWinchSystem',system.__class__.__name__)   ))
        
        steady_state = SDOFWinchSystem.from_random_data()._ode_system.solution.with_ics([0,0])[0].subs(SDOFWinchSystem.m0,10)
        display(steady_state)

        # display(Picture('./Image/steady_sol.jpg', caption = ""))


        display(ReportText('Stworzenie listy i tabelki'))
        display(ObjectCode(list_code))
        Model_number = [1,2,3,4] 
        inertia_list= [1,2,3,4] 
        stiffness_list = [1,2,3,4] 
        damping_list = [1,2,3,4] 
        Excitation_list = [1,2,3,4]

        system_parameters = {'Inertia':inertia_list, 'Stiffness':stiffness_list,'Damping':damping_list,'Excitation':Excitation_list}


        display(ObjectCode(df_code))

        df1 =  pd.DataFrame(index = Model_number, data = system_parameters)
        display(df1)

        # display(Picture('./Image/listatab.jpg', caption = ""))


        display(ReportText('Utworzenie wektora - zestaw wartosci czasu dla naszej symulacji '))
        display(Markdown('''
            function = lambdify(SDOFWinchSystem.ivar, steady_state)
            t_span = np.linspace(0,100,200)'''))

        function = lambdify(SDOFWinchSystem.ivar, steady_state)
        t_span = np.linspace(0,100,200)

        # display(Picture('./Image/wektor1.jpg', caption = ""))


        display(ReportText('Utworzenie słowników z danymi i podstawienie danych w nim zawartych do naszego rownania, tak aby bylo zalezne jedynie od czasu '))

        display(ObjectCode(dict_code))
        # display(Picture('./Image/slowniki_sym.jpg', caption = ""))


        display(ReportText('Utworzenie listy elementów podstawiajac po kolei wartosci czasu '))
        display(ObjectCode(list_elem))
        display(Picture('./dynpy/utilities/components/guides/images/eq2_fun.jpg', caption = ""))


        display(ReportText('Generowanie wykresu rownania szczegolnego '))
        display(ObjectCode(solution_code))
        display(Picture('./dynpy/utilities/components/guides/images/wykres_niefun.jpg', caption = ""))


        display(ReportText('Generowanie wykresu rownania ogolnego '))
        display(ObjectCode(sym_gen_sol))
        display(Picture('./dynpy/utilities/components/guides/images/ogolne_tab.jpg', caption = ""))
        display(Picture('./dynpy/utilities/components/guides/images/ogolne_plot.jpg', caption = ""))

class BasicOperationsComponent(ReportComponent):

    title="Podstawowe obekty i operacje"

    def append_elements(self):

        ss1 = pd.Series([2,34,5,-2,8,12])
        x1 = LatexDataFrame(ss1)

        ss2=ss1*10
        x2 = LatexDataFrame(ss2)
        ss3=ss1.abs()

        x3 = LatexDataFrame(ss3)
        ss4=ss1.describe()
        x4 = LatexDataFrame(ss4)

        s5=ss1
        s5.index =['pierwszy','drugi','trzeci','czwarty','piąty','szósty']
        x5 = LatexDataFrame(s5)

        wojewodztwa_list = ['mazowieckie','wielkopolskie','lubelskie','warmińsko-mazurskie','zachodniopomorskie','podlaskie','dolnośląskie','pomorskie','łódzkie','kujawsko-pomorskie','podkarpackie','małopolskie','lubuskie','śląskie','świętokrzyskie','opolskie']
        powierzchnia_lista=[35558,29826,25122,24173,22892,20187,19947,18310,18219,17972, 17846,15183,13988,12333,11711,9412]
        ludnosc_lista=[5349114,3475323,2139726,1439675,1710482,1188800,2904207,2307710,2493603,2086210, 2127657,3372618,1018075,4570849,1257179,996011]
        dochody_lista=[4464,3371,3338,3447,3649,3552,3788,4104,3438,3508,3212, 3395,3187,3586,3268,3112]
        wydatki_lista=[787,596,623,597,767,697,742,1023,590,778,574,598,365,631,647,431]
        wojwodztwa_dane={'powierzchnia':powierzchnia_lista,'l.osob':ludnosc_lista,'dochody':dochody_lista,'wydatki':wydatki_lista}
        
        df = pd.DataFrame(index=wojewodztwa_list,data=wojwodztwa_dane)
        x6 = LatexDataFrame(df)

        area = Symbol('A',positive=True)
        df['zaludnienie']=df['l.osob']/df['powierzchnia']
        x7 = LatexDataFrame(df)
        
        df21=df.iloc[1:3,0:3]
        x8 = LatexDataFrame(df21)
        
        df22=df.loc['mazowieckie']
        x9 = LatexDataFrame(df22)
        
        df.loc['Warszawa',:]=[4000,600000,2500,300,120]
        x10 = LatexDataFrame(df)
        
        df30=df.set_axis([1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17],axis='index')
        x11 = LatexDataFrame(df30)
        
        a,b,c,d,e = symbols('a b c d e',positive = True)

        units_dict = {a:ureg.meter,b:ureg.second, 'lubuskie':ureg.meter}
        LatexDataFrame.set_default_units(units_dict)

        df31=df.set_axis([a,b,c,d,e],axis='columns')
        x12 = LatexDataFrame(df31)
        
        df40=df.rename(columns={'l.osob':'LUDNOŚĆ'},index={'lubuskie':'LUBU...'})
        x21 = LatexDataFrame(df40)
        
        df50=df.dochody.apply(lambda x: x*2)
        x13 = LatexDataFrame(df50)
        
        df60=df.applymap(lambda x: x*2)
        x14 = LatexDataFrame(df60)
        
        #system = self.reported_object # it's useless in the case of permanent content - it's commented for future usage
        
        display(ReportText('**Importowanie biblioteki Pandas**'))
        display(ObjectCode(
            '''
            import pandas as pd
            '''))

        display(ReportText('**Tworzenie serii (analogicznie do kolumn w Excelu)**'))
        display(x1.reported())

        display(ObjectCode(
            '''
            s = pd.Series([2,34,5,-2,8,12]) 
            '''))

        display(ReportText('**Operacje na seriach**s*10 '))
        display(x2.reported())

        display(ReportText('**Funkcja abs() zmiana wartrosci na dodtanie** s.abs()'))
        display(x3.reported())

        display(ReportText(' **Funkcja describe() - podstawowe statystyki** s.describe() '))
        display(x4.reported())

        display(Markdown(
        '''
        **Zmiana nazw indeskow domyslnie zaczynajacych sie od 0**
        s.index =['pierwszy','drugi','trzeci','czwarty','piąty','szósty']
        '''))
        display(x5.reported())
        
        display(ReportText('**DataFrame (tabele) - tworzenie DataFrame na bazie słownika** *tworzenie listy wojewodztw*'))
        display(ObjectCode(
            '''
            wojewodztwa_list = ['mazowieckie','wielkopolskie','lubelskie'
            'warmińsko-mazurskie','zachodniopomorskie','podlaskie'
            'dolnośląskie','pomorskie','łódzkie'
            'kujawsko-pomorskie','podkarpackie','małopolskie'
            'lubuskie','śląskie','świętokrzyskie','opolskie']
            '''))
        
        display(ReportText('*tworzenie listy powierzchni wojewodztw w km^2*'))
        display(ObjectCode(
            '''
            powierzchnia_lista=[35558,29826,25122,24173,22892,20187,19947,18310,18219,17972,17846,15183,13988,12333,11711,9412]
            '''))

        display(ReportText('*tworzenie listy liczby ludności*'))
        display(ObjectCode(
            '''
            ludnosc_lista=[5349114,3475323,2139726,1439675,1710482,1188800,2904207,2307710,2493603,2086210,2127657,3372618,1018075,4570849,1257179,996011]
            '''))
        
        display(ReportText('*tworzenie listy dochodow na osobe*'))
        display(ObjectCode(
            '''
            dochody_lista=[4464,3371,3338,3447,3649,3552,3788,4104,3438,3508,3212,3395,3187,3586,3268,3112]
            '''))

        display(ReportText('*tworzenie listy wydatków majątkowych/inwestycyjnych na osobe*'))
        display(ObjectCode(
            '''
            wydatki_lista=[787,596,623,597,767,697,742,1023,590,778,574,598,365,631,647,431]
            '''))

        display(ReportText('*Tworzenie słownika*'))
        display(ObjectCode(
            '''
           wojwodztwa_dane={'powierzchnia':powierzchnia,'l.osob':ludnosc_lista,'doch.':dochody_lista,'wyd.':wydatki_lista}
            '''))

        display(ReportText('*Wywołanie tabeli*'))
        display(ObjectCode(
            '''
            df = pd.DataFrame(index=wojewodztwa_list,data=wojwodztwa_dane)
            df
            '''))
        display(x6.reported())

        display(ReportText('**Operacje na kolumnach - dodanie kolumny z obliczoną gęstością zaludnienia**'))
        display(ObjectCode(
            '''
            df['zalud.']=df['l.os.']/df['pow.']
            df
            '''))
        display(x7.reported())
        
        
        display(ReportText('**Metoda iloc - za pomoca indeksów wykonujemy operacje na wierszach i kolumnach**'))
        display(ObjectCode(
            '''
            df.iloc[1:3,0:3]
            '''))
        display(x8.reported())
        
        display(ReportText('**Metoda loc**'))
        display(ObjectCode(
            '''
            df.loc['mazowieckie']
            '''))
        display(x9.reported())
        
        display(ReportText('**Dodanie wiersza za pomocą metody loc**'))
        display(ObjectCode(
            '''
            df.loc['Warszawa',:]=[4000,600000,2500,300,120]
            '''))
        display(x10.reported())

        display(ReportText('**Metoda set_axis dla wierszy - operacje na indeksach**'))
        display(ObjectCode(
            '''
            df.set_axis([1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17],axis='index')
            '''))
        display(x11.reported())

        display(ReportText('**Metoda set_axis dla kolumn**'))
        display(ObjectCode(
            '''
            a,b,c,d,e = symbols('a b c d e',positive = True)
            
            units_dict = {a:ureg.meter,b:ureg.second, 'lubuskie':ureg.meter}
            LatexDataFrame.set_default_units(units_dict)

            LatexDataFrame.formatted(df.set_axis([a,b,c,d,e],axis='columns'))
            '''))
        display(LatexDataFrame.formatted(x12).reported())
        
        display(ReportText('**Metoda rename - zmienia nazwy kolumni wierszy** '))
        display(ObjectCode(
            '''
            df.rename(columns={'l.osob':'LUDNOŚĆ'},index={'lubuskie':'LUBU...'})
            '''))
        display(x21.reported())

        display(ReportText('**Metoda apply - implemetacja funkcji na wybranej kolumnie tabeli**'))
        display(ObjectCode(
            '''
            df.dochody.apply(lambda x: x*2)
            '''))
        display(x13.reported())

        display(ReportText('**Metoda applymap - implemetacja funkcji na calej tabeli**'))
        display(ObjectCode(
            '''
            df.applymap(lambda x: x*2)
            '''))
        display(x14.reported())
        
        
#ODESystem
obiekt_1=(
'''
x = dynamicsymbols('x')
omega = Symbol('\\omega',positive = True)
Omega = Symbol('\\Omega',positive = True)
t = Symbol('t')
'''
)
obiekt_2=(
'''
m = Symbol('m',positive = True)
c = Symbol('c',positive = True)
k = Symbol('k',positive = True)
t=Symbol('t')
x=dynamicsymbols('x')

eq1 = Eq(m*x.diff(t,2)+c*x.diff(t)+k*x,0) \n

odesys1 = ODESystem(odes = Matrix([eq1.lhs-eq1.rhs]),dvars = Matrix([x], ode_order=2)) \n

odesys1.solution
''')

class BasicUsageOfODESystemComponent(ReportComponent):
    
    title="Podstawy używania klasy ODESystem"


    def append_elements(self):

        display(ReportText('Klasa ODESystem służy do rozwiązywania i analizowania równań różniczkowych. Tworzy ona obiekt w postaci równania różniczkowego zwyczajnego (ODE) w formie ogólnej i zapewnia metody podstawowych operacji, takich jak rozwiązanie ogólne czy szczególne.'))

        display(ReportText(
        '''\n Do prawidłowego wykonania zadania należy przyjąć następujące kroki:

        - Definiowanie potrzebnych symboli, które wykorzystamy do budowy ODESystem: 
        '''))

        display(ObjectCode(obiekt_1))
        display(ReportText('- Definiowanie równania, które wykorzystamy do budowy ODESystem: '))

        display(ObjectCode('''eq = Eq(x.diff(t, t) + omega**2 * x,sin(Omega*t))'''))


        display(ReportText('- Definiowanie ODESystem, gdzie " odes " to przeniesione na jedną stronę równanie, które będziemy rozwiązywać (najlepiej podawać w macierzy), a " dvars " to zmienne, po których rozwiązujemy nasze równanie (równierz podajemy jako macierz):'))

        display(ObjectCode('''odesys = ODESystem(odes=Matrix([eq.lhs-eq.rhs]),dvars=Matrix([x]), ode_order=2)'''))

        display(ReportText('- Rozwiązanie ogólne wcześniej zdefiniowanego równania metodą general solution, czyli rozwiązanie ogólne:'))

        display(ObjectCode('''odesys.general_solution[0]'''))

        display(ReportText('- Rozwiązanie szczególne zdefiniowanego równania metodą steady solution, rozwiązanie szczególne'))

        display(ObjectCode('''odesys.steady_solution'''))

        display(ReportText('- Przykład wykorzystania ODESystem dla oscylatora drgań:'))

        display(ObjectCode(obiekt_2))


        
harmonic_esc_str=(
'''
harmonic_oscilator_eq = Eq(m*x.diff(t,t) -F + c*x.diff(t) + k*x,0)
harmonic_oscilator_eq  
''')

harmonic_oscylator_ode = (
'''
harmonic_oscylator_ode = ODESystem(odes= Matrix([(harmonic_oscilator_eq.lhs/m).expand()]),dvars = Matrix([x]),ode_order=2 )
harmonic_oscylator_ode
''')

harmonic_oscylator_sol = (
'''
harmonic_oscylator_ode = ODESystem(odes= Matrix([(harmonic_oscilator_eq.lhs/m).expand()]),dvars = Matrix([x]),ode_order=2 )
harmonic_oscylator_ode
''')

harmonic_oscylator_sym = (
    '''
harmonic_oscylator_sym = harmonic_oscylator_sol.subs(subs_dict).compute_solution(t_span)
harmonic_oscylator_sym
''')

ode_sys_P = (
'''ode_sys_P = ODESystem.from_dynamic_system(dyn_sys_P.linearized())
ode_sys_P
'''
)

ode_sol_P = (
'''
ode_sol_P = ode_sys_P.solution.with_ics(ics_list)
ode_sol_P
''')

num_mod_P = (
'''
 = dyn_sys_P.linearized().subs(subs_dict_P).numerized().with_ics(ics_list)
num_mod_P
''')

num_sol_P = (
'''
num_sol_P = num_mod_P.compute_solution(t_span)
''')

num_sol_str = (
'''
display(num_sol_P.plot())
''')

num_wyn_SMS = (
'''
num_wyn_SMS = ode_sys_SMS.subs(subs_dict).numerized().with_ics(ics_list).compute_solution(t_span)
''')

ana_wyn_SMS = (
'''
ana_wyn_SMS = ode_sol_SMS.subs(subs_dict).with_ics(ics_list).compute_solution(t_span)
''')

num_wyn_SMS_plot = (
'''
display(num_wyn_SMS[x].plot())
''')

ana_wyn_SMS_plot = (
'''
display(ana_wyn_SMS[x].plot())
''')


ChapterContainer_str=(
'''
dokument=Chapter'Stworzenie raportu  klasy ODESystem'
CurrentCotainer(dokument)
''')

Chapter_SympyFormula=(
'''
display(SympyFormula(nazwa_zdefiniowaneg_rownania))
''')

ode_components_path_str=(
'''
MarekSkwarka/Shared_files/modules/dynpy/utilities/components/ode/en.py
''')

property_report_formula=(
'''
doc_final = DevelopmentGuide('./Output/[PL][Guide][Programming][PL][Guide][ODESystem]Obsługa klasy ODESystem',documentclass=NoEscape('article'),document_options=['a4paper','eqn'],lmodern=False)
doc_final.packages.appened(Package('natbib',options=['numbers']))
doc_final.packages.appened(Package('booktabs'))
doc_final.packages.appened(Package('float'))
doc_final.packages.appened(Package('siunitx'))

doc_final.append(intro)
doc_final.append(dokument)
doc_final.append(komponent)
doc_final.append(symki)

doc_final.generate_pdf()
result = False
result = doc_final.generate_pdf()
''')

class ODEReportComponent(ReportComponent):
    
    title="Stworzenie raportu"


    def append_elements(self):
        
        #system = self.reported_object # it's useless in the case of permanent content - it's commented for future usage


        display(ReportText('Aby utworzyć dokument, standardowo należy utworzyć sekcje lub rozdziały w następujący sposób:'))
#        display(Picture('./dynpy/utilities/components/guides/images/dok.jpg'))
        display(ObjectCode(ChapterContainer_str))
        display(ReportText('Jeżeli chcemy żeby nasze równania ładnie wyswietlały się w dokumencie nie możemy zapominac o użyciu SympyFormula:'))
        display(ObjectCode(Chapter_SympyFormula))
#        display(Picture('./dynpy/utilities/components/guides/images/sympaj.jpg'))
        display(ReportText('Co więcej, ODESystem posiada ogromną ilosć komponentów raportujących, które tworzą sekcje raportu za nas. Scieżka do ich odnalezienia jest następująca:'))
        display(ObjectCode(ode_components_path_str)) 
#        display(Picture('./dynpy/utilities/components/guides/images/sciezka_ode.jpg'))
        display(ReportText('Co więcej, istnieje też property .report, o którym informacja znajduje się rozdział niżej. Aby wygenerować ładnego pdf z użyciem         konkretnego szablonu potrzebujemy zdefiniować nasz dokuemnt jako wybrany template a następnie standardowo zaapendować sekcje w następujący sposób:'))
        display(ObjectCode(property_report_formula)) 
#        display(Picture('./dynpy/utilities/components/guides/images/appeend.jpg'))

class ODENumericalSimulationsComponent(ReportComponent):
    
    title="Symulacje numeryczne- różne techniki symulacji"


    def append_elements(self):
        
        #system = self.reported_object # it's useless in the case of permanent content - it's commented for future usage

        display(ReportText('Przeprowadzenei symulacji przy pomocy ODE'))
        display(ReportText('W pierwszej kolejności zdefiniowano parametry potrzebne do przeprowadzenia do obliczeń'))

        display(ObjectCode(''' 
        m,F,c,k,x=symbols('m,F,c,k,x', positive=True)
        t=Symbol('t')
        x=Function(x)(t)

         subs_dict={m:100 ,
                  F:200 ,
                  c:50 ,
                  k:500 
                  }

        ics_list = [0.0,0.0]
        '''))

        display(ReportText('Wyprowadzenie wzoru na oscylator harmoniczny'))

        display(ObjectCode(harmonic_esc_str))

        display(ReportText('Przeprowadzenie symulacji w ODE'))


        display(ObjectCode(harmonic_oscylator_ode))


        display(ObjectCode(harmonic_oscylator_sol))

        display(ObjectCode)

        display(ObjectCode(harmonic_oscylator_sym))

        display(ReportText('Wprowadzenie systemu dynamicznego, zdefiniowano również pojawiającą się Omegę '))
        display(ObjectCode('''
        dyn_sys_P = PulledPendulum()
        dyn_sys_P._eoms

        subs_dict_P = dyn_sys_P.get_numerical_parameters()
        subs_dict_P = {dyn_sys_P.F: 10, dyn_sys_P.Omega: 3.14/5, **subs_dict_P}


        subs_dict_P
        ics_list_P = [0.0,0.0] # 1 stopień swobody
        '''))

        display(ObjectCode('''
        sym_wyn_P = ode_sol_P.subs(subs_dict_P).compute_solution(t_span)
        sym_wyn_P.plot()
        '''))
        display(ObjectCode(ode_sys_P))
        display(ObjectCode(ode_sol_P))
        display(ObjectCode(num_mod_P))


        display(ReportText('Wyplotowanie solvera'))
        display(ObjectCode(num_sol_str))
        display(ObjectCode(num_sol_P))

        display(ReportText('Symulacje używając systemu dynamicznego zdefiniowanego przy użyciu modułu dynpy'))


        display(ReportText('Sprawdzenie za pomocą helpa czy tak na prawdę jest NumericalAnalysisDataFrame'))
        display(ObjectCode('''
        help(NumericalAnalysisDataFrame)
        type(coords)
        '''))

        display(ReportText('Wyświetlenie poszczególnych wykresów, wyników oraz tablicy'))
        display(ObjectCode('''
        display(harmonic_oscylator_ode)

        display(harmonic_oscylator_sol)

        coords= harmonic_oscylator_sol.lhs

        nadf = NumericalAnalysisDataFrame.from_model(harmonic_oscylator_sol,F,[10, 20, 30], reference_data=subs_dict,coordinates=list(coords), index=pd.Index(t_span, name=t),ics=None)

        nadf_sol = nadf.perform_simulations()

        nadf_sol.columns = nadf_sol.columns.droplevel(0)

        nadf_sol
        '''))

        display(ObjectCode('''
        nadf_sol.plot()
        '''))

        display(ReportText('Analiza porównawcza symulacji rozwiązania analitycznego i numerycznego'))


        display(ObjectCode('''
        Omega = Symbol('Omega', positive=True)

        subs_dict={m:10,
                   F:100,
                   c:0.01,
                   k:500,
                   }

        t_span = np.linspace(1,2000,2001)

        ode_sys_SMS = harmonic_oscylator_ode.subs({F: F*sin(Omega*t)}).subs({Omega: sqrt(k/m)+0.01})

        ode_sys_SMS
        '''))


        display(ObjectCode('''
        ode_sol_SMS = ode_sys_SMS.solution
        ode_sol_SMS.with_ics(ics_list)

        '''))

        display(ObjectCode(num_wyn_SMS))


        display(ObjectCode(ana_wyn_SMS))
        display(ObjectCode(num_wyn_SMS_plot))
        display(ObjectCode(ana_wyn_SMS_plot))

        display(ObjectCode('''
        sym_comparison = pd.DataFrame(num_wyn_SMS[x])
        sym_comparison[Symbol('x(t)_a')] = ana_wyn_SMS[x]

        sym_comparison = TimeDataFrame(sym_comparison)
        '''))

        display(ObjectCode('''
        sym_comparison.to_frequency_domain().single_sided_rms().truncate(0,0.2).plot()
        '''))


        
class ProjectileExampleComponent(ReportComponent):
    
    title="Przykład użycia ODESystem na podstawie rzutu ukośnego"


    def append_elements(self):
        from dynpy.solvers.linear import ODESystem
        #system = self.reported_object # it's useless in the case of permanent content - it's commented for future usage

        m, g, v, alpha = symbols('m g v \\alpha')

        t = Symbol('t')
        x = Function('x')(t)
        y = Function('y')(t)

        dane = {m: 10, g: 9.81, alpha: pi/6, v:20}
        predkosc_x=Eq(x.diff(t),v*cos(alpha))
        predkosc_y=Eq(y.diff(t),v*sin(alpha)-g*t)

        ode_x_x=ODESystem(odes=Matrix([predkosc_x.lhs-predkosc_x.rhs]),dvars=Matrix([x]),ode_order=1)
        ode_x_y=ODESystem(odes=Matrix([predkosc_y.lhs-predkosc_y.rhs]),dvars=Matrix([y]),ode_order=1)

        ODE_x_sol=ode_x_x.subs(dane).solution.with_ics([20])
        ODE_x_sol
        
        display(ReportText('Implementacja biblioteki ODESystem'))
        display(ObjectCode(
            '''
            from dynpy.solvers.linear import ODESystem
            '''))
  
        display(ReportText('Definiowanie zmiennych i utworzenie słownika'))
        display(ObjectCode(
            '''
            m=Symbol('m',positive=True)
            g=Symbol('g')
            alfa=Symbol('\\alpha')
            v_0=Symbol('v_0')
            t=Symbol("t")
            x=Function("x")(t)
            y=Function("y")(t)

            dane={m:10,g:10,v_0:20,alfa:pi/4}
            '''))
        
        display(ReportText('Równania ruchu dla osi x oraz y'))
        display(ObjectCode(
            '''
            v_x=Eq(x.diff(t),v_0*cos(alfa))
            v_y=Eq(y.diff(t),v_0*sin(alfa)-g*t)
            '''))
        display(ReportText('Zapisanie równań za pomocą ODESystem'))
        display(ObjectCode(
            '''
            ODE_x=ODESystem(odes=Matrix([v_x.lhs-v_x.rhs]),dvars=Matrix([x]),ode_order=1)
            ODE_y=ODESystem(odes=Matrix([v_y.lhs-v_y.rhs]),dvars=Matrix([y]),ode_order=1)
            '''))

        display(ReportText('Wywołanie odpowiednio zmiennych i funkcji daje następujące rezultaty'))
        display(ObjectCode(
            '''
            ODE_x
            '''))
        display(SympyFormula(ode_x_x))
        display(ObjectCode(
            '''
            ODE_x.general_solution[0]
            '''))
        display(SympyFormula(ode_x_x.general_solution[0]))
        display(ObjectCode(
            '''
            ODE_x.steady_solution[0]
            '''))
        display(SympyFormula(ode_x_x.steady_solution[0]))
        display(ObjectCode(
            '''
            ODE_x.solution
            '''))
        display(SympyFormula(ode_x_x.solution[0]))
        display(ObjectCode(
            '''
            ODE_x.solution.subs(dane)
            '''))

        display(SympyFormula(ode_x_x.solution.subs(dane)))
        display(ObjectCode(
            '''
            ODE_x_sol=ODE_x.subs(dane).solution.with_ics([20])
            ODE_x_sol
            '''))

        display(SympyFormula(ODE_x_sol))
        display(ReportText('Ten sam wynik można osiągnąć stosując metoodę " dsolve ", jednak nie jest ona zalecana do równań różniczkowych'))
        display(ObjectCode(
            '''
            dsolve(v_y.subs(dane))
            '''))
        display(SympyFormula(dsolve(predkosc_y.subs(dane))))

        
        
class ODESimulationComponent(ReportComponent):
    
    title="Przeprowadzanie symulacji"


    def append_elements(self):
        
        #system = self.reported_object # it's useless in the case of permanent content - it's commented for future usage

        display(ReportText('To od czego standardowo powinno się zacząć to importy oraz definicja zmiennych. Potrzebne importy:'))
        display(Picture('./dynpy/utilities/components/guides/images/importyy.jpg'))
        display(ReportText('Definiowanie zmiennych:'))
        display(Picture('./dynpy/utilities/components/guides/images/definicja.jpg'))
        display(ReportText('Kolejny niezbędny krok to tworzenie obiektu ODE. Należy go wykonać w następujący sposób:'))
        display(Picture('./dynpy/utilities/components/guides/images/obiekt_ode.jpg'))
        display(ReportText('METODA 1A: Przeprowadzenie symulacji na gotowm rozwiazaniu analitycznym poprzez użycie property solution. Samo wywołanie                                 odpowiedniego rozwiązania analitycznego pokazuje nam poniższy kod:'))
        display(Picture('./dynpy/utilities/components/guides/images/solutioon.jpg'))
        display(ReportText('Następnie generujemy tabele niezbędną do symulacji w taki sposób:'))
        display(Picture('./dynpy/utilities/components/guides/images/analityczne.jpg'))
        display(ReportText('METODA 1B: Innym sposobem jest przeprowadzenie symulacji numerycznych bez obliczeń analitycznych. Potrzebny jest do tego taki                             kod: '))
        display(Picture('./dynpy/utilities/components/guides/images/analiticzne.jpg'))
        display(ReportText('METODA 2: Trzecim sposobem generowania symulacji jest użycie NumericalAnalysisDataFrame, kod wywołania prezentuje się                                     następująco:'))
        display(Picture('./dynpy/utilities/components/guides/images/analitical_.jpg'))
        display(ReportText('Aby przeprowadzić symulacje konieczne jest zastosowanie metody perform simulations. Bez zastosowania tej metody dostaniemy                               następujący wynik:'))
        display(Picture('./dynpy/utilities/components/guides/images/bez_metody.jpg'))
        display(ReportText('A z jej użyciem, wszystko powinno zadziałać i dostaniemy następujący output:'))
        display(Picture('./dynpy/utilities/components/guides/images/z_metoda.jpg'))
        display(ReportText('Dla lepszego zrozumienia zagdnień poniżej przedstawione zostało zastosowanie symulacji na konkretnym przykładzie. Skorzystamy z                           prostej klasy SpringDamperMassSystem. Na dobry początek należy zaimportować tę klasę oraz przypisać ją do zmiennej. W przeprowadzeniu                                     symulacji pomoże nam wykorzystanie metody $ode system$ w celu pozyskania równania ruchu naszego systemu dynamicznego. Aby pozyskać dane w                                 prosty sposób korzystamy z metody $get numerical parameters$ oraz wrzucamy te dane w słownik. Kod do wykonania tych czynnosci prezentuje się                             następująco: '))
        display(Picture('./dynpy/utilities/components/guides/images/przyklad_sys.jpg'))
        display(ReportText('Następnie stosując jedną z wyżej pokazanych metod, możemy stworzyć tabelę:'))
        display(Picture('./dynpy/utilities/components/guides/images/metoda1.jpg'))
        display(ReportText('Ostatnim krokiem jest wizualizacja danych, czyli stworzenie wykresu. Aby taki wykres uzyskać wystarczy wpisać kod widoczny na                             obrazku:'))
        display(Picture('./dynpy/utilities/components/guides/images/wykresoo.jpg'))

####     
omega, omega2 = symbols('omega Omega')
t=Symbol('t')
x=Function('x')(t)
y=Function('y')(t)
ode=(
'''
ODESystem(odes=Matrix([omega**2*x-sin(omega2*t)+x.diff(t,t)]),dvars=Matrix([x]),ode_order=1)
''')   
        
        
class ReportCompUseComponent(ReportComponent):
    
    title="Podstawy używania komponentu raportującego"


    def append_elements(self):
        #system = self.reported_object # it's useless in the case of permanent content - it's commented for future usage

        komponent = Chapter('Podstawy używania komponentu raportującego')
        CurrentContainer(komponent)        
        
        display(ReportText('W celu użycia komponentu raportującego należy użyć $.report$. Przykładowo wpisując *odesys.report* otrzyma się następujący output:'))
        display(ReportText('The investigated system is described by differential equations being as follows:'))
        display(ObjectCode(ode))
        display(ReportText('To solve the problem serve several methods depending on the equation type. As dynamic systems s behaviour is described by an ordinary differential equation, the variables of the equation are as follows: t [x]'))
        display(ReportText('The variables allow to study and analyse the system s dynamics varying with time.'))


class DynSysIntroComponent(ReportComponent):

    title="Wstęp do używania modułu DynSys"
    
    
    def append_elements(self):
        from dynpy.models.mechanics.tmac import SDOFWinchSystem
        
        display(ReportText('Wymagane importy bibliotek/poszczególnych klas do stworzenia raportu:'))
        display(ObjectCode(
        '''
                from sympy import *
                import numpy as np
                from pandas import *
                from sympy.physics.mechanics import init_vprinting, dynamicsymbols
                from pylatex import Document, Section, Subsection, Itemize, Package, HorizontalSpace, Description, Marker, Command 
                from pylatex.section import Paragraph, Chapter 
                from pylatex.utils import italic, NoEscape 
                from dynpy.utilities.adaptable import *
                from dynpy.utilities.templates.document import BeamerTemplate, MechanicalCase 
                from dynpy.utilities.templates.tikz import TikzCaSCStandalone
                from dynpy.utilities.report import (SystemDynamicsAnalyzer, DataPlot, AccelerationComparison, FFTComparison, ReportEntry, SimulationalBlock,               ReportText, SimulationFFT, DataStorage, Markdown, SummaryTable, Picture, SympyFormula, SymbolsDescription, DescriptionsRegistry,
                ObjectCode, CurrentContainer)
                from dynpy.solvers.linear import ODESystem
                init_vprinting()
                from dynpy.models.mechanics.tmac import SDOFWinchSystem

        '''))
        display(ReportText('Klasa umożliwiająca rozwiązanie konkretnej problematyki (w tym przypadku problematyki związanej z opisem dźwigu):'))
        display(Markdown('''
            from dynpy.models.mechanics.tmac import SDOFWinchSystem
            '''))
        display(ReportText('Ścieżka do poszukiwanej klasy na platformie CoCalc:'))
        display(Picture('./Image/sciezka_w.jpg'))

        display(ReportText('Sposób wywowałania preview klasy - tzw. podglądu:'))

        display(ObjectCode('''
            SDOFWinchSystem().preview()
            '''))

        eoms=SDOFWinchSystem()._eoms[0]
        eoms

        display(ReportText('Proces wywołowania równania ruchu za pomoca metody eoms:'))
        display(ObjectCode(
            '''
            eoms=SDOFWinchSystem()._eoms[0]
            eoms
            '''))

        display(ReportText('Wynik jest następujący:'))
        display(SympyFormula(eoms))


        eoms_eq=Eq(eoms,0)
        eoms_eq
        display(ReportText('Proces tworzenia równania z wcześniej wywołanego równania ruchu (równanie początkowo jest wywoływane jako ciąg znaków reprezentujący opis danego problemu, jednak nie jest on wówczas równaniem, które możemy zastosować do obliczeń):'))
        display(ObjectCode('''
            eoms_eq=Eq(eoms,0)
            eoms_eq
            '''))
        display(ReportText('Wynik jest następujący:'))
        display(SympyFormula(eoms))

        #display(Picture('./Image/eomseq_w.jpg'))

        slownik=SDOFWinchSystem().get_random_parameters()


        display(ReportText('Tworzenie słownika (w tym przypadku z losowymi wartościami) umożliwiający nadanie wartości zmiennym:'))
        display(ObjectCode('''
            slownik=SDOFWinchSystem().get_random_parameters()
            slownik
            '''))
        display(slownik)
        #display(Picture('./Image/slownik_w.jpg'))

        display(ReportText('Rozwiązanie ogólne równania:'))
        #display(Picture('./Image/ogolne_w.jpg'))

        solution=SDOFWinchSystem()._ode_system.solution[0]
        display(ObjectCode(
            '''
                solution=SDOFWinchSystem()._ode_system.solution[0]
                solution

            '''))
        display(SympyFormula(solution))

        display(ReportText('Rozwiązanie szczególne równania:'))



        steady_solution=SDOFWinchSystem()._ode_system.steady_solution[0]
        display(ObjectCode(
            '''
                steady_solution=SDOFWinchSystem()._ode_system.steady_solution[0]
                steady_solution

            '''))
        display(SympyFormula(steady_solution))
eoms_code=(
'''
eoms_eq=Eq(eoms,0)
eoms_eq''')
dict_code=(
'''
param_dict = system.get_numerical_parameters()
t_span = np.linspace(1,100,101)
ic_list = [0.0,0.0] ### Dla układu o jednym stopniu swobody
#param_dict = {**param_dict,system.m:system.m}
''')
params_code=(
'''
parameter = system.system_parameters()[0]
param_dict = {**param_dict,parameter:parameter}
''')

na_df_code=(
'''
na_df = dyn_sys.subs(param_dict).numerical_analysis(parameter=system.system_parameters()[0],param_span=[1,2,3], t_span = t_span)
na_df
''')
na_df_code_eoms=(
'''
na_df = dyn_sys.subs(param_dict).eoms.numerical_analysis(parameter=system.system_parameters()[0],param_span=[1,2,3], t_span = t_span)

''')

na_df_sol_code=(
'''
na_df_sol = na_df.compute_solution(t_span = t_span, ic_list = [0.0,0.0])
na_df_sol.plot()
''')


class NumericalAnalysisSimulationComponent(ReportComponent):

    title="Wykonanie symulacji z wykorzystaniem metody numerical_analysis"


    def append_elements(self):

        from ....solvers.linear import ODESystem
        
        system = self.reported_object # it's useless in the case of permanent content - it's commented for future usage
        
        eoms=system._eoms[0]
        eoms

        display(ReportText('Proces wywołowania równania ruchu za pomoca metody eoms:'))

        display(ReportText('Wynik jest następujący:'))
        
        display(ObjectCode(eoms_code))
        
        eoms_eq=Eq(eoms,0)
        eoms_eq
        
        display(SympyFormula(eoms_eq))
        
        display(ReportText('Zaprezentowane równanie pozwala sprawdzić, czy rozważany układ jest poprawnie zaimportowany do przestrzeni roboczej.'))
        
        display(ReportText('Kolejnym kluczowym elementem jest zdefiniowanie parametrów układu, wektora czasu i warunków początkowych.'))
             
        param_dict = system.get_numerical_parameters()
        t_span = np.linspace(0,100,1001)
        ic_list = [0.0,0.0] ### Dla układu o jednym stopniu swobody
        param = system.system_parameters()[0]
                
        display(ObjectCode(dict_code))
        
        display(ReportText('Następnie należy określić, który parametr ma zostać poddany analizie. Parametr ten należy wówczas zmienić w słowniku parametrów z wartości liczbowej na symbol.'))
        
        parameter = system.system_parameters()[0]
        param_dict = {**param_dict,parameter:parameter}
        
        display(ObjectCode(params_code))
        
        
        
        display(ReportText('Ostatnim etapem jest wywołanie metody numerical_analysis dla rozważanego systemu. Możemy to zrobić przy użyciu property eoms:'))
        display(ObjectCode(na_df_code_eoms))
        display(ReportText('Lub bez niej:'))
        display(ObjectCode(na_df_code))
        na_df = system.subs(param_dict).numerical_analysis(parameter=system.system_parameters()[0],param_span=[1,2,3], t_span = t_span)
        #na_df = system.subs(param_dict).numerized()
        na_df
        
        display(ReportText('Symulacje numeryczną wykonuje się w analogiczny sposób do pozostałych symulacji w następujący sposób:'))

        display(ObjectCode(na_df_sol_code))
        
        na_df_sol = na_df.compute_solution(t_span = t_span, ic_list = [100.0,0.0])
        (na_df_sol.plot())
        
        import matplotlib.pyplot as plt
        plt.show()


sim_code=(
'''
ode_simulation = ode_solution.subs(param_dict).compute_solution(t_span, ic_list = ic_list)
ode_simulation.plot()
''')

class AnalyticalSimulationComponent(ReportComponent):

    title="Wykonanie symulacji z wykorzystaniem rozwiązania analitycznego"

    
    
    def append_elements(self):

        from ....solvers.linear import ODESystem
        
        system = self.reported_object # it's useless in the case of permanent content - it's commented for future usage
        
        eoms=system._eoms[0]
        eoms

        display(ReportText('Proces wywołowania równania ruchu za pomoca metody eoms:'))

        display(ReportText('Wynik jest następujący:'))
        
        display(ObjectCode(eoms_code))
        
        eoms_eq=Eq(eoms,0)
        eoms_eq
        
        display(SympyFormula(eoms_eq))
        
        ode_system = ODESystem.from_dynamic_system(system.linearized())

        display(ReportText('Podane równanie w tym przypadku nie będzie odpowiednim wyborem. Do wykonania symulacji analitycznej należy zastosować system dynamiczny w postaci równań różniczkowych zwyczajnych, który można uzyskać za pomocą klasy ODESystem:'))
        
        display(ReportText('Wynik jest następujący:'))

        display(SympyFormula(ode_system))
        
        param_dict = system.get_numerical_parameters()
        t_span = np.linspace(1,100,101)
        ic_list = [1.0,0.0] ### Dla układu o jednym stopniu swobody
        
        
        #param_dict = {**param_dict,system.m:system.m}
        
        
        display(ObjectCode(dict_code))
        
        
        display(ReportText('Aby przeprowadzić symulację analityczną w pierwszej kolejności konieczne jest uzyskanie rozwiązania równań ruchu dla podstawionych danych subs(param_dict), co można wykonać za pomocą metod `solution` i `steady_solution` w zależności od skomplikowania systemu.'))
        
        ode_solution = ode_system.subs(param_dict).steady_solution
        
        display(ReportText('Wynik jest następujący:'))
        
        display(SympyFormula(ode_solution))
        
        display(ReportText('Do przeprowadzenia symulacji konieczne jest zdefiniowanie parametrów układu, wektora czasu oraz warunków początkowych. Można je uzyskać w następujący sposób:'))
        

        
        display(ObjectCode(sim_code))
        
        display(ReportText('Dla zdefiniowanych parametrów symulacja analityczna wygląda w następujący sposób:'))
        
        ode_simulation = ode_solution.subs(param_dict).compute_solution(t_span, ic_list = ic_list)

        (ode_simulation.plot())
        import matplotlib.pyplot as plt
        plt.show()


class DynamicSystemCompletenessCheckComponent(ReportComponent):

    title="Wywoływanie i sprawdzanie wszystkich kluczowych elementów systemu dynamicznego"


    def append_elements(self):
        
        system = self.reported_object # it's useless in the case of permanent content - it's commented for future usage
        
        eoms=system._eoms[0]
        eoms

        display(ReportText('Wywołanie systemu w pierwszej kolejności można sprawdzić przez sprawdzenie równania ruchu metodą _eoms.'))

        display(ReportText('Wynik jest następujący:'))
        
        display(Markdown(
            '''
        eoms_eq=Eq(eoms,0)
        eoms_eq
            '''))
        
        eoms_eq=Eq(eoms,0)
        eoms_eq
        
        display(SympyFormula(eoms_eq))
        
        display(ReportText('Następnie wykonuje się analizę metody __init__ klasy:'))
        
        display(ObjectCode(system.__init__))
        
        display(ReportText('Kolejnym etapem jest sprawdzenie, czy schemat oraz zdjęcie reczywiste systemu są zdefiniowane:'))
        
        display(Markdown(
            '''
        system._as_picture()
        system.preview(example = 'detail_real_name')
            '''))
        
        system._as_picture()
        system.preview(example = 'detail_real_name')
        
        display(ReportText('Kolejnym etapem jest sprawdzenie, czy zdefiniowane są słowniki z parametrami:'))
        
        display(Markdown(
            '''
        system.get_random_parameters()
        system.get_numerical_parameters()
            '''))
        
        display(ReportText('Jeśli podczas wywołania pojawia się błąd o braku tej metody lub parametry są inne niż w metodzie __init__ należy zmodyfikować klasę analogicznie do pokazanego rozwiązania:'))
        
        display(Markdown(
            '''
        
        def get_default_data(self):

            m0, k0 = self.m0, self.k0

            default_data_dict = {

                self.m: [S.One * no * m0 /100 for no in range(80, 120)], # percentage
                self.k_r: [S.One * no * k0/100 for no in range(80, 120)], # percentage
                self.k_l: [S.One * no * k0/100 for no in range(70, 110)], # percentage
            }
            return default_data_dict

        def get_numerical_data(self):


            m0, k0 = 100, 1000


            default_data_dict = {
                self.m: [S.One * no * m0 /100 for no in range(80, 120)], # percentage
                self.k_r: [S.One * no * k0/100 for no in range(80, 120)], # percentage
                self.k_l: [S.One * no * k0/100 for no in range(70, 110)], # percentage

            }
            return default_data_dict
            '''))
        
        system.get_random_parameters()
        system.get_numerical_parameters()
        
        display(ReportText('Następnie konieczne jest sprawdzenie opisów symboli oraz jednostek:'))
        
        display(Markdown(
            '''
        system.symbols_description()
        system.unit_dict()
            '''))
        
        display(ReportText('Jeśli podczas wywołania tej metody pojawia się błąd, należy wykonać modyfikację w analogizny sposób do:'))
        
                
        display(Markdown(
            '''
        def symbols_description(self):
            self.sym_desc_dict = {
                self.m: r'system's mass',
                self.k_l: r'left spring stiffness',
                self.k_r: r'left spring stiffness',
            }
            return self.sym_desc_dict

        def unit_dict(self):
        
            from pint import UnitRegistry
            ureg=UnitRegistry()

            unit_dict = {
                self.m: ureg.kilogram,
                self.k_l: ureg.Newton/ureg.meter,
                self.k_r: ureg.Newton/ureg.meter,
            }
            return unit_dict

            '''))
        
        from pint import UnitRegistry
        ureg=UnitRegistry()
        
        system.symbols_description()
        system.unit_dict()
        
        
issue_title_str = """
Maintenance of `{system_name}` class which is dynamic system representation        
"""
        
issue_desc_str = """

The following problems have to be checked or fixed, in order to ensure the correctness of implemented system:

- [ ] checking if a class is executable,

- [ ] validation of figures for schemes and real examples,

- [ ] validation of reference parameters,

- [ ] validation of random parameters,

- [ ] validation of  description of  description of parameters,

- [ ] validation of units.
"""
        
class IssuePreparationComponent(ReportComponent):
    
    title="Wzór na tworzenie issue w systemie GitHub"

    def append_elements(self):
        
        system = self.reported_object # it's useless in the case of permanent content - it's commented for future usage
        system_name = system.__class__.__name__
        
        display(ReportText("Przykładowy tekst, który pozwoli na sprawne przygotowywanie issue ma następującą formę:"))

        display(ReportText("# Title: "))
        display(ObjectCode(issue_title_str.format(system_name=system_name)))

        display(ReportText("# Description: "))
        display(ObjectCode(issue_desc_str))
        
        
        
class_call_str=(
'''
from dynpy.models.mechanics.principles import ComposedSystem
from sympy import *

class MyMaterialPointMovement(ComposedSystem):
    pass
''')

atributes_str=(
'''
m = Symbol('m', positive=True)
g = Symbol('g', positive=True)
c = Symbol('c', positive=True)
r = Symbol('r', positive=True)
phi = dynamicsymbols('phi')

c0 = Symbol('c0', positive=True)
r0 = Symbol('r0', positive=True)
phi0 = dynamicsymbols('phi0')
''')

innit_str=(
'''
def __init__(self,
             m=None,
             g=None,
             c=None,
             r=None,
             phi=None,
             ivar=Symbol('t'),
             **kwargs):

    if m is not None: self.m = m
    if g is not None: self.g = g
    if c is not None: self.c = c
    if r is not None: self.r = r
    if phi is not None: self.phi = phi
    self.ivar = ivar

    self.qs = [self.phi]

    self._init_from_components(**kwargs)
''')

component_str=(
'''
@property
def components(self):

    components = {}


    self._mass_x = MaterialPoint(self.m,
                                 pos1=self.r * sin(self.phi),
                                 qs=self.qs)
    self._mass_y = MaterialPoint(self.m,
                                 pos1=self.r * cos(self.phi),
                                 qs=self.qs)

    self._gravity_ = GravitationalForce(self.m,
                                        self.g,
                                        pos1=self.r * cos(self.phi),
                                        qs=self.qs)



    components['_mass_x']=self._mass_x
    components['_mass_y']=self._mass_y
    components['_gravity_']=self._gravity_


    return components
''')


symb_desc_str=(
'''
def symbols_description(self):
    self.sym_desc_dict = {
        self.m: r'Mass',
        self.g: r'Gravity constant',
        self.c: r'',
    }

    return self.sym_desc_dict
''')

def_data_str=(
'''
def get_default_data(self):

    m0, c0, r0, phi0 = self.m0, self.c0, self.r0, self.phi0

    default_data_dict = {
        self.m: [m0 * no for no in range(1, 8)],
        self.c: [c0 * no for no in range(1, 8)],
        self.r: [r0 * no for no in range(1, 8)],
        self.phi: [phi0 * no for no in range(1, 8)],
    }

    return default_data_dict
''')

num_data_str=(
'''
def get_numerical_data(self):

    m0, c0, r0, phi0 = self.m0, self.c0, self.r0, self.phi0

    default_data_dict = {
        self.m: [m0 * no for no in range(1, 8)],
        self.c: [c0 * no for no in range(1, 8)],
        self.r: [r0 * no for no in range(1, 8)],
        self.phi: [phi0 * no for no in range(1, 8)],
    }

    return default_data_dict
''')

dict_str=(
'''
def unit_dict(self):

    from pint import UnitRegistry
    ureg=UnitRegistry()

    unit_dict = {
        self.m: ureg.kilogram,
        self.g: ureg.meter/ureg.second/ureg.second,
        self.c: ureg.kilogram/ureg.second,
        self.r: ureg.meter,
        self.phi: ureg.radian,
        self.c0: ureg.kilogram/ureg.second,
        self.r0: ureg.meter,
        self.phi0:  ureg.radian
    }
    return unit_dict
''')

full_class_str=(
'''
from dynpy.models.mechanics.principles import ComposedSystem
from sympy import *

class MyMaterialPointMovement(ComposedSystem):

    m = Symbol('m', positive=True)
    g = Symbol('g', positive=True)
    c = Symbol('c', positive=True)
    r = Symbol('r', positive=True)
    phi = dynamicsymbols('phi')

    c0 = Symbol('c0', positive=True)
    r0 = Symbol('r0', positive=True)
    phi0 = dynamicsymbols('phi0')

    def __init__(self,
                 m=None,
                 g=None,
                 c=None,
                 r=None,
                 phi=None,
                 ivar=Symbol('t'),
                 **kwargs):

        if m is not None: self.m = m
        if g is not None: self.g = g
        if c is not None: self.c = c
        if r is not None: self.r = r
        if phi is not None: self.phi = phi
        self.ivar = ivar

        self.qs = [self.phi]

        self._init_from_components(**kwargs)

    @property
    def components(self):

        components = {}


        self._mass_x = MaterialPoint(self.m,
                                     pos1=self.r * sin(self.phi),
                                     qs=self.qs)
        self._mass_y = MaterialPoint(self.m,
                                     pos1=self.r * cos(self.phi),
                                     qs=self.qs)

        self._gravity_ = GravitationalForce(self.m,
                                            self.g,
                                            pos1=self.r * cos(self.phi),
                                            qs=self.qs)


      
        components['_mass_x']=self._mass_x
        components['_mass_y']=self._mass_y
        components['_gravity_']=self._gravity_
     

        return components

    def symbols_description(self):
        self.sym_desc_dict = {
            self.m: r'Mass',
            self.g: r'Gravity constant',
            self.c: r'',
        }

        return self.sym_desc_dict

    def get_default_data(self):

        m0, c0, r0, phi0 = self.m0, self.c0, self.r0, self.phi0

        default_data_dict = {
            self.m: [m0 * no for no in range(1, 8)],
            self.c: [c0 * no for no in range(1, 8)],
            self.r: [r0 * no for no in range(1, 8)],
            self.phi: [phi0 * no for no in range(1, 8)],
        }

        return default_data_dict

    def get_numerical_data(self):

        m0, c0, r0, phi0 = self.m0, self.c0, self.r0, self.phi0

        default_data_dict = {
            self.m: [m0 * no for no in range(1, 8)],
            self.c: [c0 * no for no in range(1, 8)],
            self.r: [r0 * no for no in range(1, 8)],
            self.phi: [phi0 * no for no in range(1, 8)],
        }

        return default_data_dict
        
    def unit_dict(self):

        from pint import UnitRegistry
        ureg=UnitRegistry()

        unit_dict = {
            self.m: ureg.kilogram,
            self.g: ureg.meter/ureg.second/ureg.second,
            self.c: ureg.kilogram/ureg.second,
            self.r: ureg.meter,
            self.phi: ureg.radian,
            self.c0: ureg.kilogram/ureg.second,
            self.r0: ureg.meter,
            self.phi0:  ureg.radian
        }
        return unit_dict        
''')

class DynSysImplementationComponent(ReportComponent):
    
    title="Implementacja systemow dynamicznych"


    def append_elements(self):
        
        #system = self.reported_object # it's useless in the case of permanent content - it's commented for future usage


        #przekopiowac tu fragment kodu z guide pamietajac o indentach
        display(ReportText("Tworzenie klasy należy zacząć od wybrania nazwy opisującej nasz system oraz zadeklarowania jej w odpowiedni sposób, który został przedstawiony poniżej:")) #itp
        display(GuideCode(class_call_str))
        display(ReportText('Następnie deklarujemy atrybuty/symbole, które są niezbędne do opisania danego obiektu'))
        display(GuideCode(atributes_str))
        display(ReportText('Do zdefiniowania inicjalizacji obiektu klasy z określonymi atrybutami służy poniższa struktura.'))
        display(GuideCode(innit_str))
        display(ReportText('W tym kroku korzystamy z predefiniowanych elementów takich jak punkty materialne, dyski, itp.'))
        display(GuideCode(component_str))
        display(ReportText('W tej części opisujemy parametry klasy, np. masę'))
        display(GuideCode(symb_desc_str))
        display(ReportText('Sekcja ta przedstawia inicjalizację słownika zawierającego dane symboliczne do równań analitycznych systemu:'))
        display(GuideCode(def_data_str))
        display(ReportText('Kolejnym krokiem jest przypisanie słowników zawierających dane do symulacji numerycznych:'))
        display(GuideCode(num_data_str))
        display(ReportText('Ta sekcja opisuje definiowanie jednostek używanych przez system dynamiczny:'))
        display(GuideCode(dict_str))
        
        display(ReportText('Ostatecznie kod implementacji całego systemu ma następującą postać:'))
        display(GuideCode(full_class_str))
        
        
        



from sympy import Symbol, symbols, Matrix, sin, cos, diff, sqrt, S, diag, Eq, Dict, ImmutableMatrix, latex, solve#, Tuple



# Amadeusz Radomski

class Reliability():
    '''
    Class bases on First Order Reliability Method. By asumming mean values and standard deviations of the parameters, the class calculates new design point parameters until reliabiity index convergence.
    '''

    def __init__(self,
                limit_state_function,
                mean_values_dict,
                standard_deviations_dict,
                parameters_list,
                dependent_variable=None,
                new_design_point_coords=None,
                ):

        self.limit_state_function = limit_state_function
        self.mean_values_dict = mean_values_dict
        self.parameters_list = parameters_list
        self.standard_deviations_dict = standard_deviations_dict

        if dependent_variable is not None: self.dependent_variable = dependent_variable
        else: self.dependent_variable = self.parameters_list[-1]

        if new_design_point_coords is not None:

            dictionary={}
            for i, param in enumerate(self.parameters_list):
                dictionary[param] = new_design_point_coords[i]

            self.new_design_point_coords = dictionary

        else:
            self.new_design_point_coords = mean_values_dict



    def dependent_variable_dict(self, iteration=None):
        '''
        Method returns dict containing calulated dependent parameter expression in original space.
        '''

        mean_val_dict = self.new_design_point_coords.copy()

        func = Eq(0, self.limit_state_function.rhs, evaluate=False)

        func_sol = (solve(func, self.dependent_variable)[0])#subs(mean_val_dict)

#         func_sol_eq = Eq(self.dependent_variable, func_sol, evaluate=False)

        mean_val_dict[self.dependent_variable] = func_sol

        return mean_val_dict

    def design_point_in_original_space(self, iteration=None):
        '''
        Method defines coordinates of the design point in the original space.
        '''

        x_star=Symbol(f'x__*',positive=True)

        mean_val_dict = self.dependent_variable_dict()

        design_point_mat = Matrix([self.parameters_list]).subs(mean_val_dict)

        eq1 = Eq(x_star, design_point_mat, evaluate=False)

        eq2 = Eq(x_star, eq1.rhs.subs(mean_val_dict).n(), evaluate=False)

        return eq2


    def design_point_in_reduced_space(self, iteration=None):
        '''
        Method defines coordinates of the design point in the reduced space.
        '''

        x_star_prim=Symbol('''x__'*''',positive=True)

        dp_in_og_space = self.design_point_in_original_space().rhs

        coords_list=[]

        for i,coord in enumerate(dp_in_og_space):

            reduced_coord = ((coord-self.parameters_list[i].subs(self.mean_values_dict))/self.parameters_list[i].subs(self.standard_deviations_dict)).n()
            coords_list.append(reduced_coord)

        design_point_mat = Matrix([coords_list])

        eq = Eq(x_star_prim, design_point_mat.T, evaluate=False)

        return eq

    def design_point_derivatives_vector(self, iteration=None):
        '''
        Method defines vector containing the derivatives of the performance function evaluated at the design point.
        '''
        A=Symbol('A',positive=True)

        mean_val_dict = self.dependent_variable_dict()

        diff_list=[]

        for i,parameter in enumerate(self.parameters_list):

            calulated_diff = (self.limit_state_function.rhs.diff(parameter).subs(mean_val_dict).subs(mean_val_dict)*parameter.subs(self.standard_deviations_dict)).n()
            diff_list.append(calulated_diff)

        diff_design_point_vector = Matrix([diff_list])

        eq = Eq(A, diff_design_point_vector.T, evaluate=False)

        return eq

    def estimate_reliability_index(self, iteration=None):
        '''
        Method estimates the reliability index.
        '''

        beta_hl=Symbol('beta_HL')

        A=self.design_point_derivatives_vector().rhs
        x_prim_star = self.design_point_in_reduced_space().rhs

        eq = Eq(beta_hl, ((-(A.T*x_prim_star)[0])/sqrt((A.T*A)[0,0])).n(), evaluate=False)

        return eq

    def vector_of_directional_cosines(self, iteration=None):
        '''
        Method estimates the vector of directional cosines.
        '''

        A=self.design_point_derivatives_vector().rhs

        alpha=Symbol('alpha',positive=True)

        directional_cosines_vector = (A/(sqrt(A.T * A))[0,0]).n()

        eq = Eq(alpha, directional_cosines_vector, evaluate=False)

        return eq

    def new_design_point_in_reduced_space(self, iteration=None):
        '''
        Method estimates the new design point in reduced space for n-1 variables.
        '''

        alpha= self.vector_of_directional_cosines().rhs
        beta_hl = self.estimate_reliability_index().rhs

        x_star_prim=Symbol('''x__'*''',positive=True)


        new_design_point_list=[]

        for i,parameter in enumerate(self.parameters_list):

            new_design_point = (-alpha[i,0]*beta_hl).n()
            new_design_point_list.append(new_design_point)

        new_design_point_vector = Matrix([new_design_point_list])

        new_design_point_vector.col_del(i)

        eq = Eq(x_star_prim, new_design_point_vector.T, evaluate=False)

        return eq

    def new_design_point_in_original_space(self, iteration=None):
        '''
        Method estimates the new design point in original space.
        '''

        new_design_point_in_reduced_space = self.new_design_point_in_reduced_space().rhs

        x_star=Symbol('''x__*''',positive=True)

        new_design_point_list=[]
        design_point_dict={}

        for i,parameter in enumerate(self.parameters_list):

            if i < len(new_design_point_in_reduced_space):
                new_design_point = (parameter.subs(self.mean_values_dict)+new_design_point_in_reduced_space[i,0]*parameter.subs(self.standard_deviations_dict)).n()
                new_design_point_list.append(new_design_point)
                design_point_dict[parameter] = new_design_point
            else:
                break


        new_design_point_vector = Matrix([new_design_point_list])

        sol = (self.dependent_variable.subs(self.dependent_variable_dict()).subs(design_point_dict)).n()

        new_design_point_vector = new_design_point_vector.col_insert(i, Matrix([[sol]]))


        eq = Eq(x_star, new_design_point_vector.T, evaluate=False)

        return eq

    def next_step(self):

        lsf=self.limit_state_function

        return Reliability(limit_state_function=lsf,
                mean_values_dict=self.mean_values_dict,
                standard_deviations_dict=self.standard_deviations_dict,
                parameters_list=self.parameters_list,
                dependent_variable=self.dependent_variable,
                new_design_point_coords=self.new_design_point_in_original_space().rhs,)


    def search_for_stable_index(self, n=None, treshold=None):

        i=0
        if n is None and treshold is None: n=5
        elif treshold is not None: n = 100

        display(f'iteracja {i+1}')

        list_for_coords = []
        list_for_param = []
        list_for_new_mdp = []

        og_space = self.design_point_in_original_space()
        display(og_space)
#         display(self.design_point_in_reduced_space())
#         display(self.design_point_derivatives_vector())
        rel_index_initial=self.estimate_reliability_index()
        display(rel_index_initial)
#         display(self.vector_of_directional_cosines())
#         display(self.new_design_point_in_reduced_space())

        new_og_space = self.new_design_point_in_original_space()
        display(new_og_space)

        list_for_coords.append(og_space.rhs)
        list_for_param.append(rel_index_initial.rhs)
        list_for_new_mdp.append(new_og_space.rhs)

        new_reliability_instance = self.next_step()

        for i in range(n-1):
            display(f'iteracja {i+2}')

            og_space = new_reliability_instance.design_point_in_original_space()
            display(og_space)
            rel_index = new_reliability_instance.estimate_reliability_index()
            display(rel_index)
            new_og_space = new_reliability_instance.new_design_point_in_original_space()
            display(new_og_space)

            list_for_coords.append(og_space.rhs)
            list_for_param.append(rel_index.rhs)
            list_for_new_mdp.append(new_og_space.rhs)

            if abs(rel_index.rhs - rel_index_initial.rhs) <= treshold:
                display(f'Różnica dwóch ostatnich indeksów niezawodności jest mniejsza bądź równa progowi równemu {treshold}')
                break

            new_reliability_instance = new_reliability_instance.next_step()
            rel_index_initial = rel_index

            i=i+1


        return {'Original space':list_for_coords, 'Estimated reliability index':list_for_param, 'New original space':list_for_new_mdp}


    def mean_values_sensivity_analysis(self, mpp, pf, beta):

        list_for_sens=[]

        for i,parameter in enumerate(self.parameters_list):

            sens_symbol = Symbol(f's_\mu {parameter}',positive=True)

            sensitivity = pf*mpp.T[0,i]/(beta*parameter.subs(self.standard_deviations_dict))

            eq=Eq(sens_symbol, sensitivity, evaluate=False)

            display(eq)

            list_for_sens.append([eq])

        return list_for_sens

    def standard_deviations_sensivity_analysis(self, mpp, pf, beta):

        list_for_sens=[]

        for i,parameter in enumerate(self.parameters_list):

            sens_symbol = Symbol(f's_\sigma {parameter}',positive=True)

            sensitivity = pf*(mpp.T[0,i])**2/(beta*parameter.subs(self.standard_deviations_dict))

            eq=Eq(sens_symbol, sensitivity, evaluate=False)

            display(eq)

            list_for_sens.append([eq])

        return list_for_sens
    
class InverseReliability(Reliability):
    '''
    Class bases on First Order Reliability Method. By asumming expected reliability index and input, the class calculates dependent variable value.
    '''

    def __init__(self,
                limit_state_function,
                mean_values_dict,
                standard_deviations_dict,
                parameters_list,
                assumed_reliability_index,
                dependent_variable=None,
                new_design_point_coords=None,
                ):

        self.limit_state_function = limit_state_function
        self.mean_values_dict = mean_values_dict
        self.parameters_list = parameters_list
        self.standard_deviations_dict = standard_deviations_dict
        self.assumed_reliability_index = assumed_reliability_index

        if dependent_variable is not None: self.dependent_variable = dependent_variable
        else: self.dependent_variable = self.parameters_list[-1]

        if new_design_point_coords is not None:

            dictionary={}
            for i, param in enumerate(self.parameters_list):
                dictionary[param] = new_design_point_coords[i]

            self.new_design_point_coords = dictionary

        else:
            self.new_design_point_coords = mean_values_dict

    def estimate_value(self, deviation=False):
        '''
        Method estimates the parameter value.
        '''

        beta_hl=Symbol('beta_HL')

        if deviation is False:
#             symbol = self.dependent_variable.subs(self.new_design_point_coords)
            symbol = self.dependent_variable.subs(self.mean_values_dict)
        else:
            symbol = self.dependent_variable.subs(self.standard_deviations_dict)

        eq = self.estimate_reliability_index()
#         display(eq)
#         display(self.new_design_point_coords)
#         display(symbol)
        sol = solve(eq, symbol)[0].subs({beta_hl: self.assumed_reliability_index})
    
#         if sol < 0:
#             sol = 0

        return Eq(symbol, sol, evaluate=False)

    def next_iteration(self):

        coords = self.new_design_point_in_original_space().rhs.subs(self.estimate_value().lhs,self.estimate_value().rhs).T

#         display(coords)
#         display(type(coords))

        new_design_point_list = []
        for i,parameter in enumerate(coords):

            if i < len(coords):
                new_design_point = coords[0,i]
                new_design_point_list.append(new_design_point)
            else:
                break

        new_design_point_vector = Matrix([new_design_point_list])

        variable_to_append = self.dependent_variable.subs(self.new_design_point_coords)

        new_design_point_vector = new_design_point_vector.col_insert(i, Matrix([[variable_to_append]]))

#         coords[-1] = self.dependent_variable.subs(self.new_design_point_coords)

        next_iteration_step = InverseReliability(limit_state_function=self.limit_state_function,
                                         mean_values_dict=self.mean_values_dict,
                                         standard_deviations_dict=self.standard_deviations_dict,
                                         parameters_list=self.parameters_list,
                                         dependent_variable=self.dependent_variable,
                                         assumed_reliability_index=self.assumed_reliability_index,
                                         new_design_point_coords=new_design_point_vector)

        return next_iteration_step


    def search_for_stable_parameter(self, n=None, treshold=None):

        i=0
        if n is None and treshold is None: n=5
        elif treshold is not None: n = 100

        display(f'iteracja {i+1}')

        list_for_coords = []
        list_for_param = []
        list_for_new_mdp = []

        og_space = self.design_point_in_original_space()

#         display(self.design_point_in_reduced_space())
#         display(self.design_point_derivatives_vector())
        param_value_initial=self.estimate_value()
        display(param_value_initial)

#         display(self.vector_of_directional_cosines())
#         display(self.new_design_point_in_reduced_space())

        new_og_space = self.new_design_point_in_original_space().rhs.subs(self.estimate_value().lhs,self.estimate_value().rhs)
        new_og_space = Eq(self.new_design_point_in_original_space().lhs, new_og_space, evaluate=False)

        list_for_coords.append(og_space.rhs)
        list_for_param.append(param_value_initial.rhs)
        list_for_new_mdp.append(new_og_space.rhs)

        display(new_og_space)

        new_inverse_reliability_instance = self.next_iteration()

        for i in range(n-1):
            display(f'iteracja {i+2}')

            og_space = new_inverse_reliability_instance.design_point_in_original_space()#.rhs.subs(inv_form_test_y.estimate_value().lhs,inv_form_test_y.estimate_value().rhs)
#             og_space = Eq(new_reliability_instance.new_design_point_in_original_space().lhs, og_space, evaluate=False)
            display(og_space)
            param_value=new_inverse_reliability_instance.estimate_value()
            display(param_value)
            new_og_space = new_inverse_reliability_instance.new_design_point_in_original_space().rhs.subs(new_inverse_reliability_instance.estimate_value().lhs,new_inverse_reliability_instance.estimate_value().rhs)
            new_og_space = Eq(new_inverse_reliability_instance.new_design_point_in_original_space().lhs, new_og_space, evaluate=False)
            display(new_og_space)

            list_for_coords.append(og_space.rhs)
            list_for_param.append(param_value.rhs)
            list_for_new_mdp.append(new_og_space.rhs)

            if abs(param_value.rhs-param_value_initial.rhs) <= treshold:
                display(f'Różnica dwóch ostatnich wartości parametrów jest mniejsza bądź równa progowi równemu {treshold}')
                break

            new_inverse_reliability_instance = new_inverse_reliability_instance.next_iteration()
            param_value_initial = param_value

            i=i+1

        return {'Original space':list_for_coords, 'Estimated parameter':list_for_param, 'New original space':list_for_new_mdp}

    def validate(self, n=None, treshold=None):
        '''
        Method validates the result for obtaining assumed reliability index.
        '''

        mean_values_dict = {}
        param_list = self.parameters_list

        for i,parameter in enumerate(param_list):

            if i < len(param_list):
                mean_values_dict[parameter] = param_list[i].subs(self.new_design_point_coords)
            else:
                break


        mean_values_dict[self.dependent_variable] = self.estimate_value().rhs



        validate_algorithm = Reliability(limit_state_function=self.limit_state_function,
                                         mean_values_dict=mean_values_dict,
                                         standard_deviations_dict=self.standard_deviations_dict,
                                         parameters_list=self.parameters_list,
                                         dependent_variable=self.dependent_variable)

        return validate_algorithm.search_for_stable_index(n=n, treshold=treshold)

class InverseReliabilityDeviation(InverseReliability):
    '''
    Class bases on First Order Reliability Method. By asumming expected reliability index and input, the class calculates dependent variable value.
    '''

    def __init__(self,
                limit_state_function,
                mean_values_dict,
                standard_deviations_dict,
                parameters_list,
                assumed_reliability_index,
                dependent_variable=None,
                new_design_point_coords=None,
                ):

        self.limit_state_function = limit_state_function
        self.mean_values_dict = mean_values_dict
        self.parameters_list = parameters_list
        self.standard_deviations_dict = standard_deviations_dict
        self.assumed_reliability_index = assumed_reliability_index

        if dependent_variable is not None: self.dependent_variable = dependent_variable
        else: self.dependent_variable = self.parameters_list[-1]

        if new_design_point_coords is not None:

            dictionary={}
            for i, param in enumerate(self.parameters_list):
                dictionary[param] = new_design_point_coords[i]

            self.new_design_point_coords = dictionary

        else:
            self.new_design_point_coords = mean_values_dict

    def estimate_value(self, deviation=True):
        '''
        Method estimates the parameter value.
        '''

        beta_hl=Symbol('beta_HL')

        if deviation is False:
#             symbol = self.dependent_variable.subs(self.new_design_point_coords)
            symbol = self.dependent_variable.subs(self.mean_values_dict)
        else:
            symbol = self.dependent_variable.subs(self.standard_deviations_dict)

        eq = self.estimate_reliability_index()
#         display(eq)
#         display(self.new_design_point_coords)
#         display(symbol)
        sol = solve(eq, symbol)[1].subs({beta_hl: self.assumed_reliability_index})
    
#         if sol < 0:
#             sol = 0

        return Eq(symbol, sol, evaluate=False)

    def next_iteration(self):

        coords = self.new_design_point_in_original_space().rhs.subs(self.estimate_value().lhs,self.estimate_value().rhs).T

#         display(coords)
#         display(type(coords))

        new_design_point_list = []
        for i,parameter in enumerate(coords):

            if i < len(coords):
                new_design_point = coords[0,i]
                new_design_point_list.append(new_design_point)
            else:
                break

        new_design_point_vector = Matrix([new_design_point_list])

        variable_to_append = self.dependent_variable.subs(self.new_design_point_coords)

        new_design_point_vector = new_design_point_vector.col_insert(i, Matrix([[variable_to_append]]))

#         coords[-1] = self.dependent_variable.subs(self.new_design_point_coords)

        next_iteration_step = InverseReliabilityDeviation(limit_state_function=self.limit_state_function,
                                         mean_values_dict=self.mean_values_dict,
                                         standard_deviations_dict=self.standard_deviations_dict,
                                         parameters_list=self.parameters_list,
                                         dependent_variable=self.dependent_variable,
                                         assumed_reliability_index=self.assumed_reliability_index,
                                         new_design_point_coords=new_design_point_vector)

        return next_iteration_step


    def search_for_stable_parameter(self, n=None, treshold=None):

        i=0
        if n is None and treshold is None: n=5
        elif treshold is not None: n = 100

        display(f'iteracja {i+1}')

        list_for_coords = []
        list_for_param = []
        list_for_new_mdp = []

        og_space = self.design_point_in_original_space()
        display(og_space)
        
        list_for_coords.append(og_space.rhs)
#         display(self.design_point_in_reduced_space())
#         display(self.design_point_derivatives_vector())
        param_value_initial=self.estimate_value()
        display(param_value_initial)
        
        list_for_param.append(param_value_initial.rhs)
#         display(self.vector_of_directional_cosines())
#         display(self.new_design_point_in_reduced_space())

        new_og_space = self.new_design_point_in_original_space().rhs.subs(self.estimate_value().lhs,self.estimate_value().rhs)
        new_og_space = Eq(self.new_design_point_in_original_space().lhs, new_og_space, evaluate=False)
        display(new_og_space)

        list_for_new_mdp.append(new_og_space.rhs)
        
        new_inverse_reliability_instance = self.next_iteration()
        
        


        for i in range(n-1):
            display(f'iteracja {i+2}')

            og_space = new_inverse_reliability_instance.design_point_in_original_space()#.rhs.subs(inv_form_test_y.estimate_value().lhs,inv_form_test_y.estimate_value().rhs)
#             og_space = Eq(new_reliability_instance.new_design_point_in_original_space().lhs, og_space, evaluate=False)
            list_for_coords.append(og_space.rhs)
            display(og_space)
            param_value=new_inverse_reliability_instance.estimate_value()
            display(param_value)
            list_for_param.append(param_value.rhs)
            new_og_space = new_inverse_reliability_instance.new_design_point_in_original_space().rhs.subs(new_inverse_reliability_instance.estimate_value().lhs,new_inverse_reliability_instance.estimate_value().rhs)
            new_og_space = Eq(new_inverse_reliability_instance.new_design_point_in_original_space().lhs, new_og_space, evaluate=False)
            display(new_og_space)
            list_for_new_mdp.append(new_og_space.rhs)

            if abs(param_value.rhs-param_value_initial.rhs) <= treshold:
                display(f'Różnica dwóch ostatnich wartości parametrów jest mniejsza od progu równego {treshold}')
                break

            new_inverse_reliability_instance = new_inverse_reliability_instance.next_iteration()
            param_value_initial = param_value

            i=i+1

        return {'Original space':list_for_coords, 'Estimated parameter':list_for_param, 'New original space':list_for_new_mdp}

    def validate(self, n=None, treshold=None):
        '''
        Method validates the result for obtaining assumed reliability index.
        '''

        mean_values_dict = {}
        param_list = self.parameters_list

        for i,parameter in enumerate(param_list):

            if i < len(param_list):
                mean_values_dict[parameter] = param_list[i].subs(self.new_design_point_coords)
            else:
                break


        mean_values_dict[self.dependent_variable] = self.estimate_value().rhs



        validate_algorithm = Reliability(limit_state_function=self.limit_state_function,
                                         mean_values_dict=mean_values_dict,
                                         standard_deviations_dict=self.standard_deviations_dict,
                                         parameters_list=self.parameters_list,
                                         dependent_variable=self.dependent_variable)

        return validate_algorithm.search_for_stable_index(n=n, treshold=treshold)

class ReliabilityAlgorithm(InverseReliability):
    '''
    Class bases on First Order Reliability Method. By asumming expected reliability index and input, the class calculates new design point parameters.
    '''

    def __init__(self,
                limit_state_function,
                mean_values_dict,
                standard_deviations_dict,
                parameters_list,
                assumed_reliability_index,
                dependent_variable=None,
                new_design_point_coords=None,
                ):

        self.limit_state_function = limit_state_function
        self.mean_values_dict = mean_values_dict
        self.parameters_list = parameters_list
        self.standard_deviations_dict = standard_deviations_dict
        self.assumed_reliability_index = assumed_reliability_index

        if dependent_variable is not None: self.dependent_variable = dependent_variable
        else: self.dependent_variable = self.parameters_list[-1]

        if new_design_point_coords is not None:

            dictionary={}
            for i, param in enumerate(self.parameters_list):
                dictionary[param] = new_design_point_coords[i]

            self.new_design_point_coords = dictionary

        else:
            self.new_design_point_coords = mean_values_dict


    @classmethod
    def from_inverse_reliability_method(cls, InverseReliability):
        return cls(InverseReliability.limit_state_function, InverseReliability.mean_values_dict, InverseReliability.standard_deviations_dict, InverseReliability.parameters_list, InverseReliability.assumed_reliability_index, InverseReliability.dependent_variable, None)


    def toFORM(self):

        self.validate(n=1)
